! =================================================================================================================================
! MODULE       : stomate_somdynamics
!
! CONTACT      : orchidee-help _at_ listes.ipsl.fr
!
! LICENCE      : IPSL (2006)
! This software is governed by the CeCILL licence see ORCHIDEE/ORCHIDEE_CeCILL.LIC
!
!>\BRIEF       Calculate soil dynamics largely following the Century model
!!	
!!\n DESCRIPTION: None
!!
!! RECENT CHANGE(S): None
!!
!! SVN		:
!! $HeadURL$ 
!! $Date$
!! $Revision$
!! \n
!_ ================================================================================================================================

MODULE stomate_som_dynamics

  ! modules used:

  USE ioipsl_para
  USE stomate_data
  USE constantes
  USE constantes_soil
  USE xios_orchidee

  IMPLICIT NONE

  ! private & public routines

  PRIVATE
  PUBLIC som_dynamics,som_dynamics_clear,nitrogen_dynamics,nitrogen_dynamics_clear

  ! Variables shared by all subroutines in this module
  
  LOGICAL, SAVE    :: firstcall_som = .TRUE.   !! Is this the first call? (true/false)
!$OMP THREADPRIVATE(firstcall_som)
  LOGICAL, SAVE    :: firstcall_nitrogen = .TRUE.   !! Is this the first call? (true/false)
!$OMP THREADPRIVATE(firstcall_nitrogen)
  ! flux fractions within carbon pools
  REAL(r_std),ALLOCATABLE,SAVE, DIMENSION(:,:,:)          :: frac_carb        !! Flux fractions between carbon pools 
                                                                              !! (second index=origin, third index=destination) 
                                                                              !! (unitless, 0-1)
!$OMP THREADPRIVATE(frac_carb)
  REAL(r_std), ALLOCATABLE,SAVE, DIMENSION(:,:)           :: frac_resp        !! Flux fractions from carbon pools to the atmosphere (respiration) (unitless, 0-1)
!$OMP THREADPRIVATE(frac_resp)



CONTAINS


!! ================================================================================================================================
!!  SUBROUTINE   : som_dynamics_clear
!!
!>\BRIEF        Set the flag ::firstcall_som to .TRUE. and as such activate sections 1.1.2 and 1.2 of the subroutine som_dynamics 
!! (see below).
!! 
!_ ================================================================================================================================
  
  SUBROUTINE som_dynamics_clear
    firstcall_som=.TRUE.
  END SUBROUTINE som_dynamics_clear


!! ================================================================================================================================
!!  SUBROUTINE   : som_dynamics
!!
!>\BRIEF        Computes the soil respiration and nutrient stocks, essentially 
!! following Parton et al. (1987).  Additional dynamics for the nitrogen pools are
!! described in the nitrogen_dynamics subroutine.
!!
!! DESCRIPTION	: The soil is divided into 3 pools, with different 
!! characteristic turnover times : active (1-5 years), slow (20-40 years) 
!! and passive (200-1500 years).\n
!! There are three types of nutrient (carbon, nitrogen) transferred into the soil:\n
!! - input to active and slow pools from litter decomposition,\n
!! - nutrient fluxes between the three pools,\n
!! - nurtient losses from the pools to the atmosphere, i.e., soil respiration.\n
!!
!! The subroutine performs the following tasks:\n
!!
!! Section 1.\n
!! The flux fractions (f) between carbon pools are defined based on Parton et 
!! al. (1987). The fractions are constants, except for the flux fraction from
!! the active pool to the slow pool, which depends on the clay content,\n
!! \latexonly
!! \input{soilcarbon_eq1.tex}
!! \endlatexonly\n
!! In addition, to each pool is assigned a constant turnover time.  No nitrogen
!! is considered in this section.\n
!!
!! Section 2.\n
!! The carbon and nitrogen inputs, calculated in the stomate_litter module, are 
!! added to the carbon and nitrogen stocks of the different pools.\n
!!
!! Section 3.\n
!! First, the outgoing flux of each pool is calculated. It is 
!! proportional to the product of the carbon stock and the ratio between the 
!! iteration time step and the residence time:\n
!! \latexonly
!! \input{soilcarbon_eq2.tex}
!! \endlatexonly
!! ,\n
!! Note that in the case of crops, the additional multiplicative factor 
!! integrates the faster decomposition due to tillage (following Gervois et 
!! al. (2008)).
!! In addition, the flux from the active pool depends on the clay content:\n
!! \latexonly
!! \input{soilcarbon_eq3.tex}
!! \endlatexonly
!! ,\n
!! Each pool is then cut from the carbon amount corresponding to each outgoing
!! flux:\n
!! \latexonly
!! \input{soilcarbon_eq4.tex}
!! \endlatexonly\n
!! Note that the total flux for both carbon and nitrogen out of the pools is
!! calculated first and grouped together.  This total material flux is then
!! partitioned between the elements and the pools based on a target CN ratio.\n
!! Second, the flux fractions lost to the atmosphere is calculated in each pool
!! by subtracting from 1 the pool-to-pool flux fractions. The soil respiration 
!! is then the summed contribution of all the pools,\n
!! \latexonly
!! \input{soilcarbon_eq5.tex}
!! \endlatexonly\n
!! Note that soil respiration only happens for the carbon pools.\n
!! Finally, each soil nutrient pool accumulates the contribution of the other pools:
!! \latexonly
!! \input{soilcarbon_eq6.tex}
!! \endlatexonly
!! The lefotver nitrogen flux that doesn't go into one of the four pools is 
!! mineralized.\n
!! Section 4.\n
!! If the flag SPINUP_ANALYTIC is set to true, the matrix A is updated following
!! Lardy (2011).
!!
!! RECENT CHANGE(S): Merge with the Nitrogen cycle, 2018.
!! 
!! MAIN OUTPUTS VARIABLE(S): carbon, resp_hetero_soil
!!
!! REFERENCE(S)   :
!! - Parton, W.J., D.S. Schimel, C.V. Cole, and D.S. Ojima. 1987. Analysis of 
!!   factors controlling soil organic matter levels in Great Plains grasslands. 
!!   Soil Sci. Soc. Am. J., 51, 1173-1179.
!! - Gervois, S., P. Ciais, N. de Noblet-Ducoudre, N. Brisson, N. Vuichard, 
!!   and N. Viovy (2008), Carbon and water balance of European croplands 
!!   throughout the 20th century, Global Biogeochem. Cycles, 22, GB2022, 
!!   doi:10.1029/2007GB003018.
!! - Lardy, R, et al., A new method to determine soil organic carbon equilibrium,
!!   Environmental Modelling & Software (2011), doi:10.1016|j.envsoft.2011.05.016
!! - S. Zaehle and A. D. Friend (2010), Carbon and nitrogen cycle dynamics in the
!!   O-CN land surface model: 1. Model description, site-scale evaluation, and 
!!   sensitivity to parameter estimates. Global Biogeochem. Cycles, 24, GB1005, 
!!   doi:10.1029/2009GB003521.
!!
!! FLOWCHART    :
!! \latexonly
!! \includegraphics[scale=0.5]{soilcarbon_flowchart.jpg}
!! \endlatexonly
!! \n
!_ ================================================================================================================================

  SUBROUTINE som_dynamics (npts, clay, silt, &
       som_input, control_temp, control_moist, veget_cov_max, drainage_pft,&
       CN_target,som, &
       resp_hetero_soil, &
       MatrixA,n_mineralisation, CN_som_litter_longterm, tau_CN_longterm)

!! 0. Variable and parameter declaration

    !! 0.1 Input variables
    
    INTEGER(i_std), INTENT(in)                            :: npts             !! Domain size (unitless)
    REAL(r_std), DIMENSION(npts), INTENT(in)              :: clay             !! Clay fraction (unitless, 0-1) 
    REAL(r_std), DIMENSION(npts), INTENT(in)              :: silt             !! Silt fraction (unitless, 0-1)
    REAL(r_std), DIMENSION(npts,ncarb,nvm,nelements), INTENT(in) :: som_input !! Amount of Organic Matter going into the SOM pools from litter decomposition \f$(gC m^{-2} day^{-1})$\f
    REAL(r_std), DIMENSION(npts,nlevs), INTENT(in)        :: control_temp     !! Temperature control of heterotrophic respiration (unitless: 0->1)
    REAL(r_std), DIMENSION(npts,nlevs), INTENT(in)        :: control_moist    !! Moisture control of heterotrophic respiration (unitless: 0.25->1)
    REAL(r_std), DIMENSION(npts,nvm), INTENT(in)          :: veget_cov_max    !! Fractional coverage: maximum share of the pixel taken by a pft 
    REAL(r_std), DIMENSION(npts,nvm), INTENT(in)          :: drainage_pft     !! Drainage per PFT (mm/m2 /dt_sechiba) (fraction of water content)   
    REAL(r_std), DIMENSION(npts,nvm,ncarb), INTENT(in)    :: CN_target        !! C to N ratio of SOM flux from one pool to another (gN m-2 dt-1)

    !! 0.2 Output variables
    
    REAL(r_std), DIMENSION(npts,nvm), INTENT(out)         :: resp_hetero_soil !! Soil heterotrophic respiration \f$(gC m^{-2} (dt_sechiba one_day^{-1})^{-1})$\f

    !! 0.3 Modified variables
    
    REAL(r_std), DIMENSION(npts,ncarb,nvm,nelements), INTENT(inout) :: som             !! SOM pools: active, slow, or passive, \f$(gC m^{2})$\f
    REAL(r_std), DIMENSION(npts,nvm,nbpools,nbpools), INTENT(inout) :: MatrixA  !! Matrix containing the fluxes between the carbon pools
                                                                                !! per sechiba time step 
                                                                                !! @tex $(gC.m^2.day^{-1})$ @endtex
    REAL(r_std), DIMENSION(npts,nvm), INTENT(inout)                 :: n_mineralisation !! net nitrogen mineralisation of decomposing SOM,
                                                                                        !! (gN/m**2/day), assumed to be NH4
    REAL(r_std), DIMENSION(npts,nvm,nbpools), INTENT(inout)         :: CN_som_litter_longterm !! Longterm CN ratio of litter and som pools (gC/gN)
    REAL(r_std), INTENT(inout)                                   :: tau_CN_longterm     !! Counter used for calculating the longterm CN ratio of SOM and litter pools (seconds)

    !! 0.4 Local variables
    REAL(r_std)                                           :: dt               !! Time step \f$(dt_sechiba one_day^{-1})$\f
    LOGICAL                                               :: l_error          !! Diagnostic boolean for error allocation (true/false)
    INTEGER(i_std)                                        :: ier              !! Check errors in netcdf call
    REAL(r_std), SAVE, DIMENSION(ncarb)                   :: som_turn         !! Residence time in SOM pools (days)
!$OMP THREADPRIVATE(som_turn)
    REAL(r_std), DIMENSION(npts,ncarb,nelements)          :: fluxtot          !! Total flux out of carbon pools \f$(gC m^{2})$\f
    REAL(r_std), DIMENSION(npts,ncarb,ncarb,nelements)    :: flux             !! Fluxes between carbon pools \f$(gC m^{2})$\f
    CHARACTER(LEN=7), DIMENSION(ncarb)                    :: soilpools_str    !! Name of the soil pools for informative outputs (unitless)
                                                                              !! mineral nitrogen in the soil (gN/m**2)
    INTEGER(i_std)                                        :: k,kk,m,j,l, ij   !! Indices (unitless)
    REAL(r_std), DIMENSION(npts,nvm,ncarb)                :: decomp_rate_soilcarbon !! Decomposition rate of the soil carbon pools (s) 
    REAL(r_std), DIMENSION(npts,ncarb)                    :: tsoilpools       !! Diagnostic for soil carbon turnover rate by pool (1/s)

!_ ================================================================================================================================

    !! printlev is the level of diagnostic information, 0 (none) to 4 (full)
    IF (printlev>=3) WRITE(numout,*) 'Entering som_dynamics' 

    dt = dt_sechiba/one_day
    IF ( firstcall_som ) THEN
    !! 1. Initializations
        l_error = .FALSE.
        ALLOCATE (frac_carb(npts,ncarb,ncarb), stat=ier)
        l_error = l_error .OR. (ier.NE.0)
        ALLOCATE (frac_resp(npts,ncarb), stat=ier)
        l_error = l_error .OR. (ier.NE.0)
        IF (l_error) THEN
           STOP 'stomate_som_dynamics: error in memory allocation'
        ENDIF

        frac_carb(:,:,:) = 0.0
        !! 1.1 Get soil "constants"
        !! 1.1.1 Flux fractions between carbon pools: depend on clay content, recalculated each time
        ! From active pool: depends on clay content
        frac_carb(:,iactive,ipassive) = active_to_pass_ref_frac + active_to_pass_clay_frac*clay(:)
        frac_carb(:,iactive,islow) = un - frac_carb(:,iactive,ipassive) - (active_to_co2_ref_frac - &
             active_to_co2_clay_silt_frac*(clay(:)+silt(:)))
    
        ! From slow pool    
        frac_carb(:,islow,ipassive) = slow_to_pass_ref_frac + slow_to_pass_clay_frac*clay(:)
        ! OCN doesn't use Parton 1993 formulation for frac_carb(:,islow,ipassive) 
        ! but the one of 1987 : ie = 0.03 .... 
        frac_carb(:,islow,iactive) = un - frac_carb(:,islow,ipassive) - slow_to_co2_ref_frac
    
        ! From passive pool
        frac_carb(:,ipassive,iactive) = pass_to_active_ref_frac
        frac_carb(:,ipassive,islow) = pass_to_slow_ref_frac

        ! From surface pool
        frac_carb(:,isurface,islow) = surf_to_slow_ref_frac

        !! 1.1.2 Determine the respiration fraction : what's left after
        ! subtracting all the 'pool-to-pool' flux fractions
        ! Diagonal elements of frac_carb are zero
        frac_resp(:,:) = un - frac_carb(:,:,isurface) - frac_carb(:,:,iactive) - frac_carb(:,:,islow) - &
             frac_carb(:,:,ipassive) 

        !! 1.1.3 Turnover in SOM pools (in days)
        !! som_turn_ipool are the turnover (in year)
        !! It is weighted by Temp and Humidity function later
        som_turn(iactive) = som_turn_iactive / one_year    
        som_turn(islow) = som_turn_islow / one_year            
        som_turn(ipassive) = som_turn_ipassive / one_year    
        som_turn(isurface) = som_turn_isurface / one_year
        
        !! 1.2 Messages : display the residence times  
        soilpools_str(iactive) = 'active'
        soilpools_str(islow) = 'slow'
        soilpools_str(ipassive) = 'passive'
        soilpools_str(isurface) = 'surface'

        WRITE(numout,*) 'som_dynamics:'
        
        IF (printlev >= 2) THEN
           WRITE(numout,*) '   > minimal SOM residence time in soil pools (d):'
           DO k = 1, ncarb ! Loop over soil pools
              WRITE(numout,*) '(1, ::soilpools_str(k)):',soilpools_str(k),' : (1, ::som_turn(k)):',som_turn(k)
              WRITE(numout,*) 'FRACCARB k=',k,' ',frac_carb(1,k,isurface),frac_carb(1,k,iactive), &
                   frac_carb(1,k,islow),frac_carb(1,k,ipassive)
           ENDDO
           
           WRITE(numout,*) '   > flux fractions between soil pools: depend on clay content'
        END IF        

        firstcall_som = .FALSE.
        
    ENDIF

    !! 1.3 Set soil respiration and decomposition rate to zero
    resp_hetero_soil(:,:) = zero
    decomp_rate_soilcarbon(:,:,:) = zero

!! 2. Update the SOM stocks with the different soil carbon and nitrogen input

    som(:,:,:,:) = som(:,:,:,:) + som_input(:,:,:,:) * dt

!! 3. Fluxes between nutrient reservoirs, and to the atmosphere (respiration) \n


    !! 3.2. Calculate fluxes

    DO m = 1, nvm ! Loop over # PFTs

      !! 3.2.1. Flux out of pools

      DO k = 1, ncarb ! Loop over SOM pools from which the flux comes (active, slow, passive)
        
        DO l = 1, nelements ! Loop over elements (Carbon, Nitrogen)
           ! Determine total flux out of pool
           fluxtot(:,k,l) = dt*som_turn(k) * som(:,k,m,l) * &
                control_moist(:,ibelow) * control_temp(:,ibelow) * decomp_factor(m)

           ! Flux from active pools depends on clay content
           IF ( k .EQ. iactive ) THEN
              fluxtot(:,k,l) = fluxtot(:,k,l) * ( un - som_turn_iactive_clay_frac * clay(:) )
           ENDIF
     
           ! Update the loss in each carbon pool
           som(:,k,m,l) = som(:,k,m,l) - fluxtot(:,k,l)

           ! Calculate diagnostic for decomposition rate of the soil carbon pools
           IF ( l == icarbon ) THEN
              decomp_rate_soilcarbon(:,m,k) = dt*som_turn(k) * &
                   control_moist(:,ibelow) * control_temp(:,ibelow) * decomp_factor(m)
              IF ( k .EQ. iactive ) THEN
                 decomp_rate_soilcarbon(:,m,k) = decomp_rate_soilcarbon(:,m,k)*( un - som_turn_iactive_clay_frac * clay(:) )
              ENDIF
           END IF

        ENDDO

        ! Fluxes towards the other pools (k -> kk)
        DO kk = 1, ncarb ! Loop over the SOM pools where the flux goes
          ! Carbon flux
          flux(:,k,kk,icarbon) = frac_carb(:,k,kk) * fluxtot(:,k,icarbon)
          ! Nitrogen flux - Function of the C stock of the 'departure' pool 
          ! and of the C to N target ratio of the 'arrival' pool
          flux(:,k,kk,initrogen) = frac_carb(:,k,kk) * fluxtot(:,k,icarbon) / & 
               CN_target(:,m,kk)
        ENDDO
     ENDDO ! End of loop over SOM pools
      
      !! 3.2.2 respiration
      !BE CAREFUL: Here resp_hetero_soil is divided by dt to have a value which corresponds to
      ! the sechiba time step but then in stomate.f90 resp_hetero_soil is multiplied by dt.
      ! Perhaps it could be simplified. Moreover, we must totally adapt the routines to the dtradia/one_day
      ! time step and avoid some constructions that could create bug during future developments.
      !
      resp_hetero_soil(:,m) = &
         ( frac_resp(:,iactive) * fluxtot(:,iactive,icarbon) + &
         frac_resp(:,islow) * fluxtot(:,islow,icarbon) + &
         frac_resp(:,ipassive) * fluxtot(:,ipassive,icarbon) + &
         frac_resp(:,isurface) * fluxtot(:,isurface,icarbon) ) / dt
      
      !! 3.2.3 add fluxes to active, slow, and passive pools
      
      DO k = 1, ncarb ! Loop over SOM pools
        som(:,k,m,icarbon) = som(:,k,m,icarbon) + &
           flux(:,iactive,k,icarbon) + flux(:,ipassive,k,icarbon) + flux(:,islow,k,icarbon) + flux(:,isurface,k,icarbon)
        som(:,k,m,initrogen) = som(:,k,m,initrogen) + &
           flux(:,iactive,k,initrogen) + flux(:,ipassive,k,initrogen) + flux(:,islow,k,initrogen) + flux(:,isurface,k,initrogen)
         n_mineralisation(:,m) = n_mineralisation(:,m) + fluxtot(:,k,initrogen) - &
               (flux(:,k,iactive,initrogen)+flux(:,k,ipassive,initrogen)+&
                flux(:,k,islow,initrogen)+flux(:,k,isurface,initrogen))
      ENDDO ! Loop over SOM pools
      
   ENDDO ! End loop over PFTs
    
 !! 4. (Quasi-)Analytical Spin-up
    
    !! 4.1.1 Finish to fill MatrixA with fluxes between soil pools
    
    IF (spinup_analytic) THEN

       DO m = 2,nvm 

          ! flux leaving the active pool
          MatrixA(:,m,iactive_pool,iactive_pool) = moins_un * &
               dt*som_turn(iactive) * &
               control_moist(:,ibelow) * control_temp(:,ibelow) * &
               ( 1. - som_turn_iactive_clay_frac * clay(:)) * decomp_factor(m)

          ! flux received by the active pool from the slow pool
          MatrixA(:,m,iactive_pool,islow_pool) =  frac_carb(:,islow,iactive)*dt*som_turn(islow) * &
               control_moist(:,ibelow) * control_temp(:,ibelow) * decomp_factor(m)

          ! flux received by the active pool from the passive pool
          MatrixA(:,m,iactive_pool,ipassive_pool) =  frac_carb(:,ipassive,iactive)*dt*som_turn(ipassive) * &
               control_moist(:,ibelow) * control_temp(:,ibelow) * decomp_factor(m)

          ! flux leaving the slow pool
          MatrixA(:,m,islow_pool,islow_pool) = moins_un * &
               dt*som_turn(islow) * &
               control_moist(:,ibelow) * control_temp(:,ibelow) * decomp_factor(m)

          ! flux received by the slow pool from the active pool
          MatrixA(:,m,islow_pool,iactive_pool) =  frac_carb(:,iactive,islow) *&
               dt*som_turn(iactive) * &
               control_moist(:,ibelow) * control_temp(:,ibelow) * &
               ( 1. - som_turn_iactive_clay_frac * clay(:) ) * decomp_factor(m)

          ! flux received by the slow pool from the surface pool
          MatrixA(:,m,islow_pool,isurface_pool) =  frac_carb(:,isurface,islow) *&
               dt*som_turn(isurface) * &
               control_moist(:,ibelow) * control_temp(:,ibelow) * decomp_factor(m)

          ! flux leaving the passive pool
          MatrixA(:,m,ipassive_pool,ipassive_pool) =  moins_un * &
               dt*som_turn(ipassive) * &
               control_moist(:,ibelow) * control_temp(:,ibelow) * decomp_factor(m)      

          ! flux received by the passive pool from the active pool
          MatrixA(:,m,ipassive_pool,iactive_pool) =  frac_carb(:,iactive,ipassive)* &
               dt*som_turn(iactive) * &
               control_moist(:,ibelow) * control_temp(:,ibelow) *&
               ( 1. - som_turn_iactive_clay_frac * clay(:) ) * decomp_factor(m)

          ! flux received by the passive pool from the slow pool
          MatrixA(:,m,ipassive_pool,islow_pool) =  frac_carb(:,islow,ipassive) * &
               dt*som_turn(islow) * &
               control_moist(:,ibelow) * control_temp(:,ibelow) * decomp_factor(m)

          ! flux leaving the surface pool
          MatrixA(:,m,isurface_pool,isurface_pool) =  moins_un * &
               dt*som_turn(isurface) * &
               control_moist(:,ibelow) * control_temp(:,ibelow) * decomp_factor(m)      

          WHERE (som(:,isurface,m,initrogen) .GT. min_stomate)
             CN_som_litter_longterm(:,m,isurface_pool) = ( CN_som_litter_longterm(:,m,isurface_pool) * (tau_CN_longterm-dt) &
                  + som(:,isurface,m,icarbon)/som(:,isurface,m,initrogen) * dt)/ (tau_CN_longterm)
          ENDWHERE

          WHERE (som(:,iactive,m,initrogen) .GT. min_stomate)
             CN_som_litter_longterm(:,m,iactive_pool)  = ( CN_som_litter_longterm(:,m,iactive_pool) * (tau_CN_longterm-dt) &
                  + som(:,iactive,m,icarbon)/som(:,iactive,m,initrogen) * dt)/ (tau_CN_longterm)
          ENDWHERE

          WHERE(som(:,islow,m,initrogen) .GT. min_stomate)
             CN_som_litter_longterm(:,m,islow_pool)    = ( CN_som_litter_longterm(:,m,islow_pool) * (tau_CN_longterm-dt) &
               + som(:,islow,m,icarbon)/som(:,islow,m,initrogen) * dt)/ (tau_CN_longterm)
          ENDWHERE

          WHERE(som(:,ipassive,m,initrogen) .GT. min_stomate)
             CN_som_litter_longterm(:,m,ipassive_pool) =  ( CN_som_litter_longterm(:,m,ipassive_pool) * (tau_CN_longterm-dt) &
                  + som(:,ipassive,m,icarbon)/som(:,ipassive,m,initrogen) * dt)/ (tau_CN_longterm)
          ENDWHERE

          IF (printlev>=4) WRITE(numout,*)'Finish to fill MatrixA'

       ENDDO ! Loop over # PFTS


       ! 4.2 Add Identity for each submatrix(7,7) 

       DO j = 1,nbpools
          MatrixA(:,:,j,j) = MatrixA(:,:,j,j) + un 
       ENDDO

    ENDIF ! (spinup_analytic)

    ! Output diagnostics
    DO k = 1, ncarb ! Loop over carbon pools
       DO ij = 1, npts
          IF (SUM(decomp_rate_soilcarbon(ij,:,k)*veget_cov_max(ij,:)) > min_sechiba) THEN
             tsoilpools(ij,k) = 1./(SUM(decomp_rate_soilcarbon(ij,:,k)*veget_cov_max(ij,:))/dt_sechiba)
          ELSE
             tsoilpools(ij,k) = xios_default_val
          END IF
       END DO
    END DO
    CALL xios_orchidee_send_field("tSoilPools",tsoilpools)

    IF (printlev>=4) WRITE(numout,*) 'Leaving som_dynamics'
    
  END SUBROUTINE som_dynamics


!! ================================================================================================================================
!!  SUBROUTINE   : nitrogen_dynamics_clear
!!
!>\BRIEF        Set the flag ::firstcall to .TRUE. 
!! 
!! 
!_ ================================================================================================================================
  
  SUBROUTINE nitrogen_dynamics_clear
    firstcall_nitrogen=.TRUE.
  END SUBROUTINE nitrogen_dynamics_clear




!! ================================================================================================================================
!!  SUBROUTINE   : nitrogen_dynamics
!!
!>\BRIEF        : Describes mineralization dynamics of nitrogen species in the
!!  soil.  Inspired by the DNDC model of Li et al (1992,2000), but very simplified 
!!  to avoid having to calculate microbe growth for the moment. Builds on the 
!!  physico-chemical reactions with some fixed assumptions.
!!
!! DESCRIPTION	: Plants can only uptake nitrogen in the form of NH4 and NO3.
!!   In the soil, nitrogen coverts between NH3, NH4, NO3, NOX, N2O, and N2,
!!   depending on things like the concentrations of each species, the pH of
!!   the soil, the temperature, the amount of oxygen present, and the soil
!!   moisture content.  This subroutine describes the dynamics between
!!   all these species, influencing the amount of nitrogen available for
!!   both plant uptake and litter decomposition.
!!
!!   Ammonium (NH4) may be the most important species, as all litter
!!   N is assumed to decompose to NH4.  NH4 can then be lost via
!!   adsorpotion to soil clays, transformed to NH3 or NO3, or uptaken
!!   by plants.
!!
!! The subroutine performs the following tasks:\n
!!
!! Section 2.\n
!! Immobilizes nitrogen, reducing the amount of nitrogen available in first
!! the ammonium (NH4) pool and then the nitrate (NO3) pool.  This amount is
!! later added back into the mineralized soil NH4 pool.  In essence, it
!! saves this quantity to be used in the decomposition routines.  Without
!! this step, all of the nitrogen could be used up in this subroutine, thus
!! preventing litter decomposition from happening. \n
!!
!! If more nitrogen is needed for decomposition than is available from both
!! the NH4 and NO3 pools, the deficit is recorded in N_support throughout 
!! the day.  This deficit is printed to the history files, but apparently
!! never used.\n
!!
!! Section 3.\n
!! The goal of this section is to calculate the volumetric fraction of
!! aneorobic microsites in the soil, which gives an idea of how much
!! aneorobic bacteria can be transforming soil nitrogen.  It depends
!! on the amount of oxygen in the soil, which depends on the concentration
!! difference of oxygen between the atmosphere and the soil, the diffusion
!! rate of oxygen in the soil, and how much oxygen is consumed by 
!! respiration in the soil.  For ideal gases, "concentration" is given by
!! the partial pressure.\n
!!
!! Section 4.\n
!! This section takes into account the losses of ammonium in the soil
!! due to adsorption onto clays, as well as the leaching of ammonium
!! and nitrate out of the system.\n
!!
!! Section 5.\n
!! Nitrification, that is, the change from ammonium (NH4+) to nitrate,
!! including effects of soil temperature, moisture, and pH, and
!! losses from NH4+ conversion to N2O and NO. \n
!!
!! Section 6.\n
!! Denitrification, that is the process of nitrate (NO3-) losing
!! oxygen atoms to eventually form N2, via the intermediate steps 
!! of NO2-, NO, and N2O.  All of this happens in the soil, and
!! depends on bacterial biomass, soil temperature, pH, and
!! soil water content.\n
!!
!! Section 7.\n
!! Calculates how much ammonium and nitrate is taken up by the plants.\n
!!
!! Section 8.\n
!! Calculates how much nitrogen is lost through emission into the
!! atmosphere through the surface of the soil.  Depends on the rate of
!! diffusion through the soil.  Nitrate emission is set to zero, but
!! the rest can pass from the soil to the atmosphere.  Emission is given
!! the lowest priority of any loss pathway, in the sense that if the 
!! amount of the species after changes due to leeching, nitrification, and
!! denitrification is less than what is calculated for the emissions, the
!! amount of emission is reduced to match.\n
!!
!! Section 9.\n
!! Update the values of the soil nitrogen pools, taking into account
!! all the losses and gains computed above.  After this, add any
!! external nitrogen inputs, like fertilizer and biological nitrogen
!! fixation (BNF).\n
!!
!! Section 10.\n
!! Write output variables.\n
!!
!! RECENT CHANGE(S): 
!! 
!! MAIN OUTPUTS VARIABLE(S): soil_n_min, mineralisation
!!
!! REFERENCE(S)   :
!! - S. Zaehle and A. D. Friend (2010), Carbon and nitrogen cycle dynamics in the
!!   O-CN land surface model: 1. Model description, site-scale evaluation, and 
!!   sensitivity to parameter estimates. Global Biogeochem. Cycles, 24, GB1005, 
!!   doi:10.1029/2009GB003521.
!! - Li C. S., J. Aber, F. Stange, K. Butterbach-Bahl, and H. Papen (2000), 
!!   A process-oriented model of N2O and NO emissions from forest soils: 
!!   1. Model development, Journal of Geophysical Research-Atmospheres, 
!!   105, 4369-4384.
!! - Li, C., S. Frolking, and T. A. Frolking (1992), A model of nitrous 
!!   oxide evolution from soil driven by rainfall events: 1. Model 
!!   structure and sensitivity, J. Geophys. Res., 97(D9), 9759–9776, 
!!   doi:10.1029/92JD00509. 
!! - Saxton, K.E., Rawls, W.J., Romberger, J.S., Papendick, R.I., 1986
!!   Estimating generalized soil-water characteristics from texture. 
!!   Soil Sci. Soc. Am. J. 50, 1031-1036
!! - Kesik, M., Ambus, P., Baritz, R., Brüggemann, N., et al.
!!   Inventories of N2O and NO emissions from European forest soils, 
!!   Biogeosciences, 2, 353-375, https://doi.org/10.5194/bg-2-353-2005, 2005. 
!! - Schmid, M., Neftel, A., Riedo, M. et al. 
!!   Process-based modelling of nitrous oxide emissions from different nitrogen sources in mown grassland
!!   Nutrient Cycling in Agroecosystems 60: 177. https://doi.org/10.1023/A:1012694218748, 2001.
!! 
!! FLOWCHART    :
!!
!_ ================================================================================================================================
  SUBROUTINE nitrogen_dynamics(npts, njsc, clay, sand, &
                               tsoil_decomp, tmc_pft, drainage_pft, runoff_pft, swc_pft, veget_max, resp_sol, &
                               som, biomass, input, pH, &
                               mineralisation, pb, plant_uptake, Bd,soil_n_min, &
                               p_O2,bact, N_support, &
                               cn_leaf_min_2D, cn_leaf_max_2D, cn_leaf_init_2D, &
                               mcs_hydrol, mcfc_hydrol,croot_longterm)
    !
    ! 0 declarations
    !

    ! 0.1 input

    INTEGER(i_std), INTENT(in)                                         :: npts     ! Domain size
    INTEGER(i_std),DIMENSION (npts), INTENT (in)                   :: njsc     ! Index of the dominant soil textural class in the grid cell (1-nscm, unitless)
    REAL(r_std), DIMENSION(npts), INTENT(in)                           :: clay     ! clay fraction (between 0 and 1)
    REAL(r_std), DIMENSION(npts), INTENT(in)                           :: sand     ! sand fraction (between 0 and 1)
    REAL(r_std), DIMENSION(npts), INTENT(in)                           :: tsoil_decomp        !! Temperature used for decompostition in soil (K)
    REAL(r_std), DIMENSION (npts,nvm), INTENT(in)                      :: tmc_pft             !! Total soil water per PFT (mm/m2) 
    REAL(r_std), DIMENSION (npts,nvm), INTENT(in)                      :: drainage_pft        !! Drainage per PFT (mm/m2)   
    REAL(r_std), DIMENSION(npts,nvm), INTENT(in)                       :: runoff_pft      ! ! Runoff per PFT (mm/m2 /dt_sechiba)   
                                                                                   ! (fraction of water content)
    REAL(r_std), DIMENSION (npts,nvm), INTENT(in)                      :: swc_pft             !! Relative Soil water content [tmcr:tmcs] per pft (-)  

    REAL(r_std), DIMENSION(npts,nvm), INTENT(in)                       :: veget_max! fraction of a vegetation (0-1)
    REAL(r_std), DIMENSION(npts,nvm), INTENT(in)                       :: resp_sol ! carbon respired from below ground 
                                                                                   ! (hetero+autotrophic) (gC/m**2/day)
    REAL(r_std), DIMENSION(npts,ncarb,nvm,nelements), INTENT(in)       :: som      ! SOM (gC(or N)/m**2)
    REAL(r_std), DIMENSION(npts,nvm,nparts,nelements), INTENT(in)      :: biomass  ! Biomass pools (gC(or N)/m**2)
    ! ninput=4
    REAL(r_std), DIMENSION(npts,nvm,ninput), INTENT(in)                    :: input    ! nitrogen inputs into the soil  (gN/m**2/day)
                                                                                   !  NH4 and NOX from the atmosphere, NH4 from BNF,
                                                                                   !   agricultural fertiliser as NH4/NO3 
    REAL(r_std), DIMENSION(npts), INTENT(in)                           :: pH       ! soil pH
    REAL(r_std), DIMENSION(npts,nvm), INTENT(in)                       :: mineralisation ! net nitrogen mineralisation of decomposing SOM
                                                                                         !   (gN/m**2/day), assumed to be NH4
    REAL(r_std), DIMENSION(npts), INTENT(in)                           :: pb             !! Air pressure (hPa)
    !!!!!!!! NEVER USED?
    REAL(r_std), DIMENSION(npts), INTENT(in)                           :: Bd             !! Bulk density (kg/m**3)
    !!!!!!!!
    REAL(r_std),DIMENSION(npts,nvm), INTENT(in)  :: cn_leaf_min_2D      !! minimal leaf C/N ratio 
    REAL(r_std),DIMENSION(npts,nvm), INTENT(in)  :: cn_leaf_max_2D      !! maximal leaf C/N ratio 
    REAL(r_std),DIMENSION(npts,nvm), INTENT(in)  :: cn_leaf_init_2D     !! initial leaf C/N ratio 
    REAL(r_std),DIMENSION(npts,nvm), INTENT(in)  :: croot_longterm       !! "Long term" (default 3 years) root carbon mass 
                                                                         !! per ground area  
                                                                         !! @tex $(gC m^{-2} year^{-1})$ @endtex   
    REAL(r_std),DIMENSION (nscm), INTENT(in)     :: mcs_hydrol             !! Saturated volumetric water content output to be used in stomate_soilcarbon
    REAL(r_std),DIMENSION (nscm), INTENT(in)     :: mcfc_hydrol            !! Volumetric water content at field capacity output to be used in stomate_soilcarbon
 

    REAL(r_std), DIMENSION(npts,nvm,nnspec),INTENT(inout)              :: soil_n_min      !! mineral nitrogen in the soil (gN/m**2) 
    REAL(r_std), DIMENSION(npts,nvm),INTENT(inout)                     :: p_O2            !! partial pressure of oxygen in the soil (hPa)
                      
    REAL(r_std), DIMENSION(npts,nvm),INTENT(inout)                     :: bact            !! denitrifier biomass (gC/m**2)


    ! 0.2 output

    REAL(r_std), DIMENSION(npts,nvm,nionspec), INTENT(out)             :: plant_uptake   !! Uptake of soil N by plants
                                                                                         !! (gN/m**2/timestep) 
    REAL(r_std), DIMENSION(npts,nvm), INTENT(out)                      :: N_support      !! Nitrogen which is added to the ecosystem to support vegetation growth
    ! 0.3 local
    REAL(r_std)                                           :: dt               !! Time step \f$(dt_sechiba one_day^{-1})$\f
    LOGICAL                                                            :: l_error        !! Diagnostic boolean for error allocation (true/false)
    INTEGER(i_std)                                                     :: ier            !! Check errors in netcdf call
    REAL(r_std), DIMENSION(npts,nvm,nionspec)   :: leaching    !! mineral nitrogen leached from the soil 
                                                               !! (gN/m**2/timestep)
    REAL(r_std), DIMENSION(npts,nvm)            :: immob       !! N immobilized (gN/m**2/day)
    REAL(r_std), DIMENSION(npts)                :: afps_max    !! maximum pore-volume of the soil
                                                               !! Table 2, Li et al., 2000
                                                               !! (fraction)                       
    REAL(r_std), DIMENSION(npts,nvm)            :: afps        !! pore-volume of the soil
                                                               !! Table 2, Li et al., 2000
                                                               !! (fraction)   
    REAL(r_std), DIMENSION(npts,nvm)            :: D_s         !! Oxygen diffusion in soil (m**2/day)
    REAL(r_std), DIMENSION(npts,nvm)            :: mol_O2_resp !! Density of Moles of O2 related to respiration term 
                                                               !! ((molesO2 m-3) (gC m-2)-1)
    REAL(r_std), DIMENSION(npts,nvm)            :: p_O2_resp   !! O2 partial pressure related to the respiration term 
                                                               !! ((hPa) (gC m-2)-1)
    REAL(r_std), DIMENSION(npts)                :: p_O2air     !! Oxygen partial pressure in air (hPa)
    REAL(r_std), DIMENSION(npts)                :: g_O2        !! soil gradient of O2 partial pressure (hPa m-1)
    REAL(r_std), DIMENSION(npts)                :: d_O2        !! change in O2 partial pressure (hPa day-1)
    REAL(r_std), DIMENSION(npts,nvm)            :: anvf        !! Volumetric fraction of anaerobic microsites 
                                                               !! (fraction of pore-volume)
    REAL(r_std), DIMENSION(npts)                :: FixNH4      !! Fraction of adsorbed NH4+ (-)
    REAL(r_std), DIMENSION(npts,nvm)            :: n_adsorbed  !! Ammonium adsorpted (gN/m**2)
                                                               !! based on Li et al. 1992, JGR, Table 4
    REAL(r_std), DIMENSION(npts,nvm)            :: fwnit,fwdenit          !! Effect of soil moisture on nitrification (-)
                                                               !! Zhang et al. 2002, Ecological Modelling, appendix A, page 101
   REAL(r_std), DIMENSION(npts)                :: var_temp_sol!! Temperature function used for calc. ft_nit (-)
                                                               !! Zhang et al. 2002, Ecological Modelling, appendix A, page 101
    REAL(r_std), DIMENSION(npts)                :: ft_nit      !! Effect of temperature on nitrification (-)
                                                               !! Zhang et al. 2002, Ecological Modelling, appendix A, page 101
    REAL(r_std), DIMENSION(npts)                :: fph         !! Effect of pH on nitrification (-)
                                                               !! Zhang et al. 2002, Ecological Modelling, appendix A, page 101
    REAL(r_std), DIMENSION(npts)                :: ftv         !! Effect of temperature on NO2 or NO production 
                                                               !! during nitrification (-)
                                                               !! Zhang et al. 2002, Ecological Modelling, appendix A, page 102
    REAL(r_std), DIMENSION(npts,nvm,3)          :: nitrification !! N-compounds production (NO3, N2O, NO) related 
                                                                 !! to nitrification process (gN/m**2/tstep)
    REAL(r_std), DIMENSION(npts)                :: ft_denit      !! Temperature response of relative growth rate of 
                                                                 !! total denitrifiers (-)
                                                                 !! Eq. 2 Table 4 of Li et al., 2000
    REAL(r_std), DIMENSION(npts)                :: fph_no3       !! Soil pH response of relative growth rate of 
                                                                 !! NO3 denitrifiers (-)
                                                                 !! Eq. 2 Table 4 of Li et al., 2000
    REAL(r_std), DIMENSION(npts)                :: fph_no        !! Soil pH response of relative growth rate of 
                                                                 !! NO denitrifiers (-)
                                                                 !! Eq. 2 Table 4 of Li et al., 2000
    REAL(r_std), DIMENSION(npts)                :: fph_n2o       !! Soil pH response of relative growth rate of 
                                                                 !! N2O denitrifiers (-)
                                                                 !! Eq. 2 Table 4 of Li et al., 2000
    REAL(r_std), DIMENSION(npts,nvm)                                 :: Kn_conv         !! Conversion from kgN/m3 to gN/m2
    REAL(r_std), DIMENSION(npts)                :: mu_NO3          !! Relative growth rate of NO3 denitrifiers (hour**-1)
                                                                   !! Eq.1 Table 4 Li et al., 2000 
    REAL(r_std), DIMENSION(npts)                :: mu_N2O          !! Relative growth rate of N2O denitrifiers (hour**-1)
                                                                   !! Eq.1 Table 4 Li et al., 2000
    REAL(r_std), DIMENSION(npts)                :: mu_NO           !! Relative growth rate of NO denitrifiers (hour**-1)
                                                                   !! Eq.1 Table 4 Li et al., 2000  
    REAL(r_std), DIMENSION(npts)                :: sum_n           !! sum of all N species in the soil (gN/m**2)
    REAL(r_std), DIMENSION(npts,nvm,3)          :: denitrification !! N-compounds consumption (NO3, N2O, NO) related 
                                                                   !! to denitrificaiton process (gN/m**2/tstep)
    REAL(r_std), DIMENSION(npts)                :: dn_bact     !! denitrifier biomass change (kgC/m**3/timestep)
    REAL(r_std), DIMENSION(npts)                :: ft_uptake   !! Temperature response of N uptake by plants (-)
    REAL(r_std)                                 :: conv_fac_vmax!! Conversion from (umol (gDW)-1 h-1) to (gN (gC)-1 timestep-1)
    REAL(r_std), DIMENSION(npts,nvm)            :: nc_leaf_min !! Minimal NC ratio of leaf (gN / gC)
    REAL(r_std), DIMENSION(npts,nvm)            :: nc_leaf_max !! Maximal NC ratio of leaf (gN / gC)
    REAL(r_std), DIMENSION(npts)                :: lab_n           !! Labile nitrogen in plants (gN/m**2) 
    REAL(r_std), DIMENSION(npts)                :: lab_c           !! Labile carbon in plants (gC/m**2) 
    REAL(r_std), DIMENSION(npts)                :: NCplant         !! NC ratio of the plant (gN / gC)
                                                                   !! Eq. (9) p. 3 of SM of Zaehle & Friend, 2010
    REAL(r_std), DIMENSION(npts)                :: f_NCplant       !!  Response of Nitrogen uptake by plants 
                                                                   !! to N/C ratio of the labile pool
    REAL(r_std)                                 :: conv_fac_concent!! Conversion factor from (umol per litter) to (gN m-2) 
    REAL(r_std), DIMENSION(npts)                :: frac_nh3        !! dissociation of [NH3] to [NH4+] (-)
    REAL(r_std), DIMENSION(npts)                :: F_clay          !! Response of N-emissions to clay fraction (-)
    REAL(r_std), DIMENSION(npts,nvm,nnspec)     :: emission    !! volatile losses of nitrogen 
                                                               !! (gN/m**2/timestep)
    INTEGER(i_std)                              :: m           !! Index to loop over the nvm PFT's
    REAL(r_std), DIMENSION(npts,nvm)            :: f_drain     !! fraction of tmc which has been drained in the last time step 
    REAL(r_std), DIMENSION(npts)                :: temp_sol    !! soil temperature (C)
 !_ ================================================================================================================================

    !! printlev is the level of diagnostic information, 0 (none) to 4 (full)
    IF (printlev>=3) WRITE(numout,*) 'Entering nitrogen_dynamics' 
    IF(printlev>=4)THEN
       WRITE(numout,*) 'CHECK values in nitrogen dynamics'
       WRITE(numout,*) 'soil_n_min ',soil_n_min(test_grid,test_pft,:)
    ENDIF

    !! 1. Initializations
    dt = dt_sechiba/one_day

    IF ( firstcall_nitrogen ) THEN
        firstcall_nitrogen = .FALSE.
    ENDIF

    ! Transform tsoil_decomp into degree C
    temp_sol(:) = tsoil_decomp - tp_00

    IF(printlev>=4)THEN
       WRITE(numout,*) 'CHECK values in after init'
       WRITE(numout,*) 'soil_n_min ',soil_n_min(test_grid,test_pft,:)
    ENDIF

    IF(printlev>=4)THEN
       WRITE(numout,*) 'CHECK values in after leaching'
       WRITE(numout,*) 'leaching(test_grid,test_pft,iammonium) ',leaching(test_grid,test_pft,iammonium)
       WRITE(numout,*) 'leaching(test_grid,test_pft,initrate) ',leaching(test_grid,test_pft,initrate)
       WRITE(numout,*) 'drainage_pft(test_grid) ',drainage_pft(test_grid,test_pft)
    ENDIF


    IF(printlev>=4)THEN
       WRITE(numout,*) 'PFT=',test_pft       
       WRITE(numout,*) 'leaching(test_grid,test_pft,iammonium) ',leaching(test_grid,test_pft,iammonium)
       WRITE(numout,*) 'leaching(test_grid,test_pft,initrate) ',leaching(test_grid,test_pft,initrate)
    ENDIF


    IF(printlev>=4)THEN
       WRITE(numout,*) 'drainage_pft(test_grid,test_pft) ',drainage_pft(test_grid,test_pft)
    ENDIF

    
    !! 2. Preparation for decomposition
    ! 
    ! 2.1 conservation of mass from decomposition
    ! immobilisation has absolute priority to avoid mass conservation problems
    ! the code in litter and soilcarbon has to make sure that soil ammonium can never be 
    ! more than exhausted completely by immobilisation!!!
    ! As a reminder, mineralisation is the amount of litter destined to become soil NH4
    ! 
    ! 
    immob(:,:) = zero
    WHERE(mineralisation(:,:).LT.0.)
       immob(:,:) = - mineralisation(:,:)
       ! In case, the N related to immobilisation is higher that the N
       ! available in the [NH4+] pool, we take the remaining N from the 
       ! [NO3-] pool
       soil_n_min(:,:,initrate) = soil_n_min(:,:,initrate) - &
            MAX(0.0,immob(:,:)-(soil_n_min(:,:,iammonium)-min_stomate))
  
       soil_n_min(:,:,iammonium) = soil_n_min(:,:,iammonium) - &
            MIN(immob(:,:),soil_n_min(:,:,iammonium)-min_stomate)
    ENDWHERE


    ! 2.2 Deficit
    ! In case of soil_n_min(:,:,initrate) negative, we add nitrogen and we take memory of the
    ! added nitrogen in N_support
    ! This happens when immob(:,:) is bigger than the nitrogen that can be given by the two 
    ! soil_n_min pools (nitrate and ammonium) and so soil_n_min(:,:,initrate) becomes negative.
    N_support(:,:) = 0.
    WHERE(soil_n_min(:,:,initrate) .LT. 0.)
       N_support(:,:) = - soil_n_min(:,:,initrate)
       soil_n_min(:,:,initrate) = 0.
    ENDWHERE


    IF(printlev>=4)THEN
       WRITE(numout,*) 'CHECK values after mineralisation'
       WRITE(numout,*) 'soil_n_min ',soil_n_min(test_grid,test_pft,:)
    ENDIF
  
    !! 3. ANVF 
    !
    ! The goal of this section is to calculate the volumetric fraction of
    ! aneorobic microsites (ANVF) in the soil, which gives an idea of how much
    ! aneorobic bacteria can be transforming soil nitrogen.  It depends
    ! on the oxygen diffusion and partial pressure in the soil.  These euqations
    ! are taken from Table 2 of Li et al. (2000) Table 2, though some things 
    ! remain unclear or undefinied.  afps, for example, is never explicitly given.
    ! S. Zaehle defined it with the following equation - but did not use it.
    ! In OCN, diffusion is not of afps/afps_max ratio but accounts for soilhum and 
    ! soil temperature according to Monteith & Unsworth, 1990 
    ! d_ox(:) = 1.73664 * ( 0.15 * (exp(-(soilhum_av(:)**3.)/0.44)-exp(-1./0.44))) * & 
    !          (1.+0.007*tsoil_av(:))
  
    ! The equation for afpsmax, the maximum pore volume of the soil, 
    ! is taken from Table 2 of Saxton (1986)
    afps_max(:) = h_saxton + j_saxton * (sand(:)*100.) + k_saxton * log10(clay(:)*100.)
  
    DO m=1,nvm
       ! pore volume of the soil.  Unclear where this comes from.
       afps(:,m)   = MAX(0.1,( un - swc_pft(:,m) ) * afps_max(:))

       ! diffusion through the soil. Table 2 of Li et al. (2000)
       D_s(:,m) = D_air * ( afps(:,m)**diffusionO2_power_1 ) / ( afps_max(:)**diffusionO2_power_2 )

  
       ! Account for the impact of frost on diffusion. Table 2 of Li et al. (2000)
       WHERE ( temp_sol(:) .GT. zero )
          D_s(:,m) = D_s(:,m) * F_nofrost
       ELSEWHERE
          D_s(:,m) = D_s(:,m) * F_frost
       ENDWHERE

       ! Equation (3) - Oxygen partial pressure
       ! Written in an odd differential form in Table 2 of Li et al (2000), with the
       ! time derivative of the partial pressure being related to the depth derivative
       ! of both the partial pressure and the diffusion rate.
      
       ! So we do it this way, calculating the depth gradient afterwards.
       ! Oxygen loss from respiration has to be expressed as a partial pressure
       ! using the ideal gas PV=nRRT relationship with P in Pa, V in m-3 and T in K
       ! RR is the ideal gas constat (J mol-1 K-1) and n the number of moles
       ! Respiration, one mole of C requires two moles of oxygen (CO2)
       ! Resp_below expressed in gC m-2 day-1
       ! mol_O2_resp : Density of Moles of O2 related to respiration term (molesO2 m-3) (gC m-2)-1
       ! In OCN, afps_max is used instead of afps. We keep here the original formulation
       mol_O2_resp(:,m) = (un / C_molar_mass * 2. ) / (zmaxh * afps(:,m))
       ! O2 partial pressure related to the respiration term (hPa) (gC m-2)-1
       p_O2_resp(:,m)   = mol_O2_resp(:,m) * RR * ( temp_sol(:) + tp_00 ) * Pa_to_hPa
    ENDDO
    
  
    ! Change in oxygen partial pressure d_O2 - Ds x gradient
    ! Not sure that we should use z_decomp (could be a fraction of zmaxh, too)
    ! oxygen partial pressure in air - p_O2air (hPa)
    p_O2air(:)= V_O2 * pb(:) 

    DO m = 1, nvm

       ! assume the partial pressure of oxygen in the soil is uniform over the
       ! whole depth that soil decomposers are active, in order to calculate
       ! the change in partial pressure of oxygen in the soil due to the
       ! pressure different between the air and soil and the diffusion rate
       ! of oxygen through the soil
       g_O2(:) = ( p_O2air(:) - p_O2(:,m) ) / z_decomp
       d_O2(:) = D_s(:,m) / z_decomp * g_O2(:) 
       ! D_s / z_decomp has the unit of a conductivity (m day-1)
  
       ! The new partial pressure of oxygen in the soil depends on how
       ! much oxgen has entered or left the soil due to the pressure difference
       ! between the atmosphere and the soil, in addition to any losses from
       ! soil respiration consuming oxygen to create CO2
       p_O2(:,m) = p_O2(:,m) + d_O2(:)*dt - p_O2_resp(:,m)*resp_sol(:,m)*dt
       ! Equation (4) Volumetric fraction of anaerobic microsites (ANVF)
       ! a and b constants are not specified in Li et al., 2000
       ! S. Zaehle used a=0.85 and b=1 without mention to any publication
       ! a_anvf=0.85
       ! b_anvf=1.
       anvf(:,m) = a_anvf * ( 1 - b_anvf * p_O2(:,m) / p_O2air(:) )
       anvf(:,m) = MAX(zero, anvf(:,m))
    ENDDO

    IF(printlev>=4 .AND. m==test_pft)THEN
       WRITE(numout,*) 'CHECK values after nitrification nh4 to no3'
       WRITE(numout,*) 'PFT=',test_pft
       WRITE(numout,*) 'anvf(:,m)=',anvf(test_grid,test_pft)
       WRITE(numout,*) 'p_O2(:,m)=',p_O2(test_grid,test_pft)
       WRITE(numout,*) 'p_O2air(:)=',p_O2air(test_grid)
       WRITE(numout,*) 'pb(:)=',pb(test_grid)
       WRITE(numout,*) 'd_O2(:)=',d_O2(test_grid)
       WRITE(numout,*) 'p_O2_resp(:)=',p_O2_resp(test_grid,:)
       WRITE(numout,*) 'mol_O2_resp(:)=',mol_O2_resp(test_grid,:)
       WRITE(numout,*) 'resp_soil(:,m)=',resp_sol(test_grid,test_pft)
       WRITE(numout,*) 'afps(:)=',afps(test_grid,:)
        
    ENDIF
  
    !! 4. Physical removal of NH4 and NO3
    !
    ! 4.1 Adsorption of NH4 into clay
    !
    ! Reduce soil ammonium concentrations according to the equations in
    ! Table 4 of Li et al. (1992).  This represents adsorption onto soil clays.
    ! FixNH4 : Fraction of adsorbed NH4+
    ! FixNH4=[0.41-0.47 log(NH4) clay/clay_max]
    ! NH4+ concentration in the soil liquid, gN kg-1 soil (p. 9774 of Li et al., 1992)
    ! but Zhang et al. (2002) Appendix A/B define NH4 as
    ! NH4+ in a soil layer in kgN ha-1
    ! In OCN, NH4 seems defined as kgN kg-1 or m-3 of water 
    ! Comment of N. Vuichard: I don't know which definition is the good one ...
    DO m = 1, nvm
       WHERE(soil_n_min(:,m,iammonium).GT.min_stomate)
          FixNH4(:) = MAX(0.,(a_FixNH4 + b_FixNH4 * &
               MAX(0.,log10(soil_n_min(:,m,iammonium)*10 )))) &
               * MIN(clay(:),clay_max) / clay_max
       ELSEWHERE
          FixNH4(:) = 0.0   
       ENDWHERE
       ! In OCN, we do not multiply by FixNH4 but by FixNH4/(1+FixNH4)
       ! Comment of N. Vuichard: It is not clear if we should keep the formulation of OCN or not
       ! So far, we keep the original formulation
       n_adsorbed(:,m) = MIN(soil_n_min(:,m,iammonium),soil_n_min(:,m,iammonium) * FixNH4(:))
    ENDDO
    soil_n_min(:,:,iammonium) = soil_n_min(:,:,iammonium) - n_adsorbed(:,:) 

    IF(printlev>=4)THEN
       WRITE(numout,*) 'CHECK values after adsorption'
       WRITE(numout,*) 'soil_n_min ',soil_n_min(test_grid,test_pft,:)
    ENDIF


    ! 4.2 Leaching of NH4 and NO3
    ! Initializations
    leaching(:,:,:) = zero 

    f_drain(:,:) = zero
    

    WHERE((tmc_pft(:,:)+runoff_pft(:,:)) .NE. 0) 
    	f_drain(:,:) = (fracn_drainage*drainage_pft(:,:)+fracn_runoff*runoff_pft)/(tmc_pft(:,:)+runoff_pft(:,:))
    ENDWHERE
    f_drain(:,:) = MAX(zero, MIN(f_drain(:,:),un))

    DO m = 1, nvm
       !leaching
       leaching(:,m,iammonium) = MIN(soil_n_min(:,m,iammonium) * f_drain(:,m), &
            soil_n_min(:,m,iammonium) )
       leaching(:,m,initrate)  = MIN(soil_n_min(:,m,initrate)  * f_drain(:,m), &
            soil_n_min(:,m,initrate)  )
    ENDDO

  
    !! 5. Nitrification of NH4 in the oxygenated part of the soil
    !
    ! Nitrification refers to ammonium (NH4+) being oxidized to 
    ! nitrate (NO3-) under aeroboic (i.e., in the prescence of
    ! oxygen) conditions.  This sections thus reduces the
    ! concentration of ammonium in the soil.  These equations
    ! are taken from Zhang et al. (2002), Appendix A
    ! 
    ! 5.1 Effect of environmental factors (Temp, Humidity, pH)
    ! 5.1.1 Effect of soil moisture on nitrification
    fwnit(:,:) = fwnit_0 + fwnit_1 * swc_pft(:,:) + fwnit_2 * swc_pft(:,:)**2 + fwnit_3 * swc_pft(:,:)**3 &
         + fwnit_4 * swc_pft(:,:)**4
    fwnit(:,:) = MAX (0., MIN( un, fwnit(:,:) ) )
  
    ! 5.1.2 Effect of temperature on nitrification
    ! Note that Zhang et al 2002 fold in the factor 0.1 with the parameter
    ! for the term linear in temperature (ft_nit_1), while we group it 
    ! with the soil temperature as per the rest of the terms.  Checking 
    ! the parameter values, this leads to a first impression that they 
    ! differ by a factor of 10, when in reality the same overall result 
    ! is calculated.
    var_temp_sol(:) = temp_sol(:) * 0.1
    ft_nit(:) = ft_nit_0 + ft_nit_1 * var_temp_sol(:) + ft_nit_2 * var_temp_sol(:)**2 + ft_nit_3 * var_temp_sol(:)**3 &
         + ft_nit_4 * var_temp_sol(:)**4
    ft_nit(:) = MAX (0.001, MIN( un, ft_nit(:) ) )
  
    ! 5.1.3 Effect of pH on nitrification
    fph(:) = fph_0 + fph_1 * pH(:) + fph_2 * ph(:)**2
    fph(:) = MAX(0.0, fph(:) )
  
    ! 5.1.4 Effect of temperature on NO2 or NO production during nitrification
    ! I am not sure why the factor of 0.0025*NO3,N seems to be missing from
    ! the equation reported in the literature.
    ftv(:) = ftv_0 **(ftv_1 - ftv_2 /(temp_sol(:) + tp_00) )
    ftv(:) = MAX(0.0, ftv(:) )
  
    DO m = 1, nvm

       ! 5.2 Actual nitrification rate per PFT NH4 soil pool - NH4 to NO3
       ! 
       ! Equation for the nitrification rate probably from Schmid et al., 2001, Nutr. Cycl. Agro (eq.1)
       ! but the environmental factors used are from Zhang (2002) who used an other equation
       ! I don't know how this can be mixed together ?
       ! In addition, the formulation mixed the one from Schmid with the use of the anaerobic balloon defined by Li et al., 2000. I don't know how this can be mixed together ?
       ! Last, in OCN, the default fraction of N-NH4 which is converted to N-NO3 appears equal to 2 day-1 (a factor "2" in the equation) - In Schmid et al., the nitrification rate at 20 ◦C and field capacity (knitrif,20) was set to 0.2 d−1 (Ten time less...)
       ! k_nitrif = 0.2
       nitrification(:,m,i_nh4_to_no3) = MIN(fwnit(:,m) * fph(:) * ft_nit(:) * k_nitrif * dt * & 
            soil_n_min(:,m,iammonium) ,soil_n_min(:,m,iammonium)- leaching(:,m,iammonium))
       
       IF(printlev>=4 .AND. m == test_pft)THEN
          WRITE(numout,*) 'CHECK values after nitrification nh4 to no3'
          WRITE(numout,*) 'PFT=',m
          WRITE(numout,*) 'nitrification(:,m,i_nh4_to_no3) ',nitrification(test_grid,m,i_nh4_to_no3)
          WRITE(numout,*) 'fwnit=',fwnit(test_grid,m)
          WRITE(numout,*) 'fph=',fph(test_grid)
          WRITE(numout,*) 'ft_nit=',ft_nit(test_grid)
          WRITE(numout,*) 'anvf=',anvf(test_grid,m)
       ENDIF
       
       !
       ! 5.3 Emission of N2O during nitrification - NH4 to N2O
       ! 
       nitrification(:,m,i_nh4_to_n2o) = ftv(:) * swc_pft(:,m) * n2o_nitrif_p * &  
            nitrification(:,m,i_nh4_to_no3)
       
       IF(printlev>=4 .AND. m==test_pft)THEN
          WRITE(numout,*) 'CHECK values after nitrification nh4 to n2o'
          WRITE(numout,*) 'nitrification(:,m,i_nh4_to_n2o) ',nitrification(test_grid,m,i_nh4_to_n2o)
          WRITE(numout,*) 'ftv=',ftv(test_grid)
          WRITE(numout,*) 'swc_pft=',swc_pft(test_grid,m)
       ENDIF
       
       nitrification(:,m,i_nh4_to_n2o) = MIN(nitrification(:,m,i_nh4_to_no3),nitrification(:,m,i_nh4_to_n2o))       
       !
       ! 5.4 Production of NO during nitrification - NH4 to NO
       nitrification(:,m,i_nh4_to_no) = ftv(:) * no_nitrif_p * nitrification(:,m,i_nh4_to_no3)
       
       
       IF(printlev>=4 .AND. m == test_pft)THEN
          WRITE(numout,*) 'CHECK values after nitrification nh4 to no'          
          WRITE(numout,*) 'nitrification(:,m,i_nh4_to_no) ',nitrification(test_grid,m,i_nh4_to_no)
       ENDIF
       ! 5.5 Production of NO from chemodenitrification
       ! Chemodenitrification is the conversion of NO2 to NO through a purely
       ! chemical process, dependent on temperature, soil pH, and concentration of
       ! NO2.  This is the final step of the process NH4+ -> NO3- -> NO2- -> NO
       ! Based on Chem_NO in Kesik et al.(2005)
       ! BUT Kesik et al. used NO2 concentration and not NO3 production as it is done in OCN
       ! and modification of a multiplicative constant from 300 (Kesik) to 30 (OCN)
       ! without clear motivation - I don't how this is reliable 
       ! [Matt also questions this equation...as it is written below, it is the amount
       ! of NH4 being converted to NO3 multiplied by the decomposition of NO2 to NO, but
       ! there is no NO3 to NO2 conversion included, which implies that all NO3 is
       ! instantaneously converted to NO2...which is nonsense based on the fact that an NO3
       ! pool exists]
       nitrification(:,m,i_nh4_to_no) = nitrification(:,m,i_nh4_to_no) + &
            ( chemo_0 * chemo_1 * exp(chemo_ph0 * pH(:) ) * & 
            exp(chemo_t0/((temp_sol(:)+tp_00)*RR))) * nitrification(:,m,i_nh4_to_no3)
       
       nitrification(:,m,i_nh4_to_no) = MIN(nitrification(:,m,i_nh4_to_no3)-nitrification(:,m,i_nh4_to_n2o), &
            nitrification(:,m,i_nh4_to_no))
       
       
       IF(printlev>=4 .AND. m == test_pft)THEN
          WRITE(numout,*) 'CHECK values after nitrification nh4 to no chemodenitrification'
          WRITE(numout,*) 'nitrification(:,m,i_nh4_to_no) ',nitrification(test_grid,m,i_nh4_to_no)
          WRITE(numout,*) 'pH(:)=',pH(test_grid)
          WRITE(numout,*) 'temp_sol(:)=',temp_sol(test_grid)
       ENDIF
       
       ! In OCN, NO production and N2O production is deducted from NO3 production due to NH4
       nitrification(:,m,i_nh4_to_no3) = nitrification(:,m,i_nh4_to_no3) - &
            (nitrification(:,m,i_nh4_to_no) + nitrification(:,m,i_nh4_to_n2o))
    ENDDO
     
  
    !! 6. Denitrification processes
    !
    ! Denitrification is the conversion of any oxidized N comound (NO2-, NO, N2O)
    ! to the final product of gaseous nitrogen (N2).
    !
    ! Denitrifiers are organisisms responsible for denitrification (bacteria?).
    !
    ! Li et al, 2000, JGR Table 4 eq 1, 2 and 4, ignoring NO2 (similar to NO)
    ! Comment of S. Zaehle: "includes treatment of bacteria dynamics, but I do not have any confidence in this;
    ! at least it provides rates that appear resonable." 
  
    ! 6.1 Temperature response of relative growth rate of total denitrifiers
    ft_denit(:) = 2.**((temp_sol(:)-22.5)/10.)
  
    ! 6.2 Soil pH response of relative growth rate of total denitrifiers
    ! See also comment about parenthesis' position in Thesis of Vincent Prieur (page 50)
    fph_no3(:) = 1.0 - 1.0 / ( 1.0 + EXP((pH(:)-fph_no3_0)/fph_no3_1)) 
    fph_no(:) = 1.0 - 1.0 / ( 1.0 + EXP((pH(:)-fph_no_0)/fph_no_1)) 
    fph_n2o(:) = 1.0 - 1.0 / ( 1.0 + EXP((pH(:)-fph_n2o_0)/fph_n2o_1))
  
    ! Half saturation value of N oxides (kgN/m3), Kn
    ! Unit conversion factor from kgN/m3 to gN/m2
    ! OCN used the value 0.087, but 0.083 is listed in Li et al (2000).
    ! In OCN, we account for max_eau_eau. Not clear if this is correct or not.
    Kn_conv(:,:) = Kn * tmc_pft(:,:)    
    
    ! Relative growth rate of Nox denitrifiers
    ! Eq.1 Table 4 Li et al., 2000 - but there is an error because Eq. 1 it does not 
    ! account for Temp and ph responses.  
    ! In OCN it does not account for [DOC], though Li et al. (2000) does.  We keep
    ! the formulation from OCN.  In addition, in OCN the term mu_nox(max) is missing 
    ! in the equation that defines the relative growth rate of total denitrifiers 
    ! (dn_bact) - we add the term back in here.
    !
    ! 6.3  Maximum Relative growth rate of Nox denitrifiers (hour-1): mu_no3, mu_no2, mu_no
    
    denitrification(:,:,:)=zero

    DO m = 1, nvm
    
       WHERE((tmc_pft(:,m)/(zmaxh*1000.)) .LT. mcfc_hydrol(njsc(:)))
          ! verifier zah + unite m ou mm
          fwdenit(:,m) = fwdenitfc * &
               exp(kfwdenit * (mcfc_hydrol(njsc(:))-tmc_pft(:,m)/(zmaxh*1000.))/mcfc_hydrol(njsc(:)) )
       ELSEWHERE
          fwdenit(:,m) = fwdenitfc + (1 - fwdenitfc) * &
               (tmc_pft(:,m)/(zmaxh*1000.) - mcfc_hydrol(njsc(:))) / (mcs_hydrol(njsc(:))-mcfc_hydrol(njsc(:)))
       ENDWHERE

       WHERE((soil_n_min(:,m,initrate) + Kn_conv(:,m)) .GT. min_stomate)
          mu_no3(:) = mu_no3_max * soil_n_min(:,m,initrate) / (soil_n_min(:,m,initrate) + Kn_conv(:,m))
       ELSEWHERE
          mu_no3(:) = zero
       ENDWHERE
       WHERE((soil_n_min(:,m,inox) + Kn_conv(:,m)*fact_kn_no) .GT. min_stomate)
          mu_no(:)  = mu_no_max  * soil_n_min(:,m,inox) / (soil_n_min(:,m,inox) + Kn_conv(:,m)*fact_kn_no)
       ELSEWHERE
          mu_no(:) = zero 
       ENDWHERE
       
       WHERE((soil_n_min(:,m,initrous) + Kn_conv(:,m)*fact_kn_n2o) .GT. min_stomate)
          mu_n2o(:) = mu_n2o_max * soil_n_min(:,m,initrous) / (soil_n_min(:,m,initrous) + Kn_conv(:,m)*fact_kn_n2o)
       ELSEWHERE
          mu_n2o(:) = zero
       ENDWHERE
  
       ! stocks of all N species (NO3, NO, N2O) (gN/m**2)
       sum_n(:) = soil_n_min(:,m,inox) + soil_n_min(:,m,initrous) + & 
            soil_n_min(:,m,initrate)
    
       WHERE(sum_n(:).GT.min_stomate)

          ! This appears to be an assumption that the denitrifier biomass is
          ! 0.05% of the active carbon soil organic matter.  Unclear where
          ! this comes from.
!          bact(:,m) = 0.0005*som(:,iactive,m,icarbon)
!          bact(:,m) = 0.00005*som(:,iactive,m,icarbon)
          bact(:,m) = cte_bact*som(:,iactive,m,icarbon)
          ! 6.4 Consumption rate of N oxides
          ! Table 4 Li et al.(2000)
          ! Based on the maximum growth rate on N oxides (Y_nox), maintainance
          ! coefficient on N oxides (M_nox), denitrifier biomass (bact), 
          ! relative growth rate of NOX denitrifiers (mu_nox), concentration
          ! of NOX in the soil (soil_n_min), total concentration of all nitrogen
          ! species in the soil (sum_n), the amount of NOX leached out of the
          ! soil (leaching).  The fwdenit, ft_denit, and fph_nox do not come
          ! from Table 4.  Conversion from (per hour) to (per timestep) is 
          ! handled by 24*dt at the end.
          !
          ! The reactions are NO3 -> (NO + NO2) -> N2O -> N2
          !
          ! NO3 consumption
          ! In OCN, multiplication by 0.1 of the NO3 consumption - no justification
          ! This is not kept in the current version
          denitrification(:,m,i_no3_to_nox) = MIN(soil_n_min(:,m,initrate)-leaching(:,m,initrate), &
               fwdenit(:,m) * ft_denit(:) * fph_no3(:) * ( mu_no3(:) / Y_no3 + M_no3 * soil_n_min(:,m,initrate) / sum_n(:)) * &
               bact(:,m) * 24. * dt )
          !
          ! NO consumption
          denitrification(:,m,i_nox_to_n2o) = MIN(soil_n_min(:,m,inox), &
               fwdenit(:,m) * ft_denit(:) * fph_no(:) * ( mu_no(:) / Y_no + M_no * soil_n_min(:,m,inox) / sum_n(:)) * &
               bact(:,m) * 24. * dt )        
          !
          ! N2O consumption
          denitrification(:,m,i_n2o_to_n2) = MIN(soil_n_min(:,m,initrous), &
               fwdenit(:,m) * ft_denit(:) * fph_n2o(:) * ( mu_n2o(:) / Y_n2o + M_n2o * soil_n_min(:,m,initrous) / sum_n(:)) &
               * bact(:,m) * 24. * dt )
  
          ! Dynamcis on denitrifier bacterial population change is not used, due to
          ! lack of documention in OCN and lack of clarity in Li et al (2000).
          ! In addition, OCN used dn_bact, but bact is used here.  Not clear what
          ! the consequences of this are.

       ENDWHERE
  
    ENDDO
  
    ! 
    ! 7. Plant uptake of ionic nitrogen
    !
    ! This section comes from Section 3 of the supporting material of Zaehle & Friend (2010).
    ! Only ammonium and nitrate are uptaken by plants.
    ! 
    ! Comment from OCN : Temperature function of uptake is similar to SOM decomposition
    ! to avoid N accumulation at low temperatures 
    ! In addition in the SM of Zaehle & Friend, 2010 P. 3 it is mentioned
    ! "The uptake rate is observed to be sensitive to root temperature which is included as f(T), 
    ! thereby following the temperature sensitivity of net N mineralization"
    ft_uptake(:) = control_temp_func (npts, temp_sol(:)+tp_00)
  
  
    ! Comment of N. Vuichard: Plenty of parameter values used in the formulation of the plant uptake are 
    ! not traceable to anything. I tend to rely much of the param values to the values reported 
    ! in the reference publications
    
    ! Vmax of nitrogen uptake (in umol (g DryWeight_root)-1 h-1)
    ! 
    ! In OCN, the same values are used both for uptake of NH4+ and NO3-
    ! See p. 3 of the SM of Zaehle & Friend, 2010:
    ! "As a first approximation average values for vmax, kNmin and KNmin are assumed for all PFTs
    ! (Table S1), and for both ammonium and nitrate"
    ! However based on the two papers of Kronzucker et al. (1995, 1996), it seems that vmax should be
    ! much higher for NH4+ than for NO3- . We still use the same here for both NH4+ and NO3- as in OCN
    ! The value of Vmax reported in Table S1 of Zaehle & Friend, 2010 is 5.14 ugN (g-1C) d-1
    ! Comment of N. Vuichard : I can not relate the value of vmax in OCN expressed in 
    ! (umol (g DryWeight_root)-1 h-1) (vmax=3) to the one reported in Table S1 :  5.14 ugN (g-1C) d-1
    ! The use of the conv_fac conversion factor used in OCN should help to convert (umol (g DryWeight_root)-1 h-1)
    ! in (gN (g-1C) tstep-1). conv_fac is defined as 24. * dt / 2. / 1000000. * 14. 
    ! 24 conversion of hour to day
    ! dt conversion of day to dt 
    ! 1/2 conversion of g(DryWeight) to gC
    ! 1000000. conversion of ug to g
    ! 14 conversion of umol to ugN
    ! So using conv_fac, vmax expressed in ugN (g-1C) d-1 should be equal to 
    ! 3*24./2.*14. = 504 ugN (gC)-1 d-1 not 5.14
    ! In addition, I think there is an error in the conversion from (gDW)-1 to (gC)-1
    ! When expressed per gC of root, the N uptake should be twice more than the one expressed per gDW of root,
    ! not twice less. So one should multiply by 2., not divide by 2. (so vmax = 2016 ug (gC)-1 d-1 )
    !
    ! Vmax of nitrogen uptake (in umol (g DryWeight_root)-1 h-1)
    ! vmax_uptake(iammonium) = 3. 
    ! vmax_uptake(initrate) = 3.
  
    ! Conversion from (umol (gDW)-1 h-1) to (gN (gC)-1 timestep-1)
    conv_fac_vmax= 24. * dt * 2. / 1000000. * 14. 
  
    ! minimal and maximal NC ratio of leaf (gN / gC)
    ! used in the response of N uptake to NC ratio
    ! see eq. 10 , p. 3 of SM of Zaehle & Friend, 2010
    nc_leaf_max(:,1)     = 0.
    nc_leaf_max(:,2:nvm) = 1. / cn_leaf_min_2D(:,2:nvm)
  
    nc_leaf_min(:,1)     = -1./min_stomate
    nc_leaf_min(:,2:nvm) = 1. / cn_leaf_max_2D(:,2:nvm)
  
  
    DO m = 2, nvm
       ! NCplant (gN / gC)
       ! See equation (9) p. 3 of SM of Zaehle & Friend, 2010
       lab_n(:) = biomass(:,m,ilabile,initrogen) + &
            biomass(:,m,iroot,initrogen) + biomass(:,m,ileaf,initrogen)
       lab_c(:) = biomass(:,m,ilabile,icarbon) + &
            biomass(:,m,iroot,icarbon) + biomass(:,m,ileaf,icarbon)
       WHERE(lab_c(:).GT.min_stomate)
          NCplant(:) = lab_n(:) / lab_c(:)
       ELSEWHERE
          NCplant(:) = fcn_root(m)/cn_leaf_init_2D(:,m) 
       ENDWHERE
       ! Nitrogen demand responds to N/C ratio of the labile pool for each PFT
       ! zero uptake at a NC in the labile pool corresponding to maximal leaf NC 
       ! and 1. for a NC corresponding to a minimal leaf NC
       ! See equation (10) p. 3 of SM of Zaehle & Friend, 2010
       f_NCplant(:)=min(max( ( nc_leaf_max(:,m) - NCplant(:) ) / ( nc_leaf_max(:,m) - nc_leaf_min(:,m) ), 0. ), 1.) 
  
  
       ! Plant N uptake (gN m-2 tstep-1)
       !
       ! See eq. (8) p. 3 of SM of Zaehle & Friend, 2010
       ! In OCN, there was a multiplicative factor "2" that I can not explain
       ! It may partly compensate the error mentioned above about conv_fac_vmax
       !
       ! Coefficient K_N_min (umol per litter)
       !
       ! It corresponds to the [NH4+] (resp. [NO3-]) for which the Nuptake equals vmax/2.
       ! Kronzucker, 1995 reports values that range between 20 and 40 umol for NH4+ uptake
       ! OCN seems to use 30 umol for both NH4+ and NO3-. The value used is 0.84 expressed in (gN m-2)
       ! In Kronzucker, 1995 and 1996, the concentrations are always expressed in umol. No clear 
       ! reference about the standard volume it is related to (litter ?)
       ! Coefficient K_N_min (umol per litter)
       ! K_N_min(:) = 30
       ! Conversion factor from (umol per litter) to (gN m-2) 
       conv_fac_concent = 14.0 * 1e3 * 1e-6 * 2.0
       ! 14   : molar mass for N (gN mol-1)
       ! 10^3 : conversion factor (dm3 to m3)
       ! 10-6 : conversion factor (ug to g)
       ! 2    : m3 per m2 of soil
       !
       ! Comment of N. Vuichard : I wonder why it assumes 2 m3 per m2 of soil. The depth of the soil 
       ! is 2 meter. But it doesn't mean that all the column contains only water, there is also soil, no ?
       ! The question is : what is the volume that is considered for the concentration of NH4+ and NO3-. Is it a
       ! volume of soil or of solution ? 
       ! If it is a volume of solution, I would suggest to multiply by a factor corresponding to the relative volume 
       ! of water within the soil. - Not done yet. 
       ! 
       ! K_N_min * conv_fac_concent = 0.84
       !
       ! Coefficient low_K_N_min ((gN m-2)-1)
       !
       ! See eq. 8 of SM of Zaehle et al. (2010) and Table S1
       ! In table S1, it is defined as 
       ! "Rate of N uptake not associated with Michaelis- Menten Kinetics" with a value of 0.05
       ! It is mentioned also (unitless) but to my opinion (N. Vuichard) it should have
       ! the same unit that 1/K_N_min or 1/N_min, so ((gN m-2)-1)
       ! Comment of N. Vuichard -------------------------------------------------------------- 
       ! So far, I cannot relate the value of low_K_N_min (0.05) to any reference 
       ! especially Kronzucker (1996)
       ! If I refer to Figure 4 of Kronzucker showing the NH4+ influx as a function of NH4+ 
       ! concentration, the slope of the relationship could be used to define Vmax*low_K_N_min
       ! For a concentration of NH4+ of 50 mmol, the influx equals 35 umol g-1 h-1
       ! For a concentration of NH4+ of 20 mmol, the influx equals 17 umol g-1 h-1
       ! slope = 10-3 * (35 - 17) / (50 - 20) = 10-3 * 18 / 30 = 0.0006 g-1 h-1 
       ! low_K_N_min = slope / Vmax = 0.0006 / 3 = 0.0002 (umol)-1 
       ! using the Conversion factor from (umol per litter) to (gN m-2) 
       ! conv_fac_concent = 14 * 1e3 * 1e-6 * 2, one should get 
       ! low_K_N_min = 0.0002 / ( 14 * 1e3 * 1e-6 * 2 ) = 0.007 ((gN m-2)-1)
       ! The value of 0.007 does not match with the one in OCN (0.05) - This needs to be clarified
       ! End of comment -----------------------------------------------------------------------
       ! low_K_N_min ((umol)-1)
  
       ! In eq. 8, I (N. Vuichard) think there is an error. It should not be
       ! (Nmin X KNmin) but (Nmin + KNmin). The source code of OCN is correct     
       plant_uptake(:,m,iammonium) = vmax_uptake(iammonium) * conv_fac_vmax * croot_longterm(:,m) &
            * ft_uptake(:) * soil_n_min(:,m,iammonium) * ( low_K_N_min(iammonium) / conv_fac_concent &
            + 1. / ( soil_n_min(:,m,iammonium) + K_N_min(iammonium) * conv_fac_concent ) ) * f_NCplant(:)
  
       plant_uptake(:,m,initrate) = vmax_uptake(initrate) * conv_fac_vmax * croot_longterm(:,m) &
            * ft_uptake(:) * soil_n_min(:,m,initrate) * ( low_K_N_min(initrate) / conv_fac_concent &
            + 1. / ( soil_n_min(:,m,initrate) + K_N_min(initrate) * conv_fac_concent ) ) * f_NCplant(:)


       IF ((printlev>=4).AND.(m==test_pft)) THEN
          WRITE(numout,*) 'plant_uptake ',plant_uptake(test_grid,test_pft,iammonium)
          WRITE(numout,*) 'vmax_uptake(iammonium) ',vmax_uptake(iammonium)
          WRITE(numout,*) ' biomass(:,m,iroot,icarbon)', biomass(test_grid,m,iroot,icarbon)
          WRITE(numout,*) ' ft_uptake(test_grid)',ft_uptake(test_grid)
          WRITE(numout,*) 'soil_n_min ',soil_n_min(test_grid,test_pft,iammonium)
          WRITE(numout,*) 'f_NCplant(test_grid)', f_NCplant(test_grid)
          WRITE(numout,*) 'NCplant(test_grid)', NCplant(test_grid)
          WRITE(numout,*) 'nc_leaf_max(m)',nc_leaf_max(test_grid,m)
          WRITE(numout,*) 'nc_leaf_min(m)',nc_leaf_min(test_grid,m)
          WRITE(numout,*) 'lab_c(test_grid)',lab_c(test_grid)
          WRITE(numout,*) 'lab_n(test_grid)',lab_n(test_grid)
          WRITE(numout,*) 'leaching ',leaching(test_grid,test_pft,iammonium)
          WRITE(numout,*) 'nitrification ',nitrification(test_grid,test_pft,:)
          WRITE(numout,*) 'tmc_pft',tmc_pft(test_grid,test_pft)
       ENDIF

       
       plant_uptake(:,m,iammonium) = MIN(soil_n_min(:,m,iammonium) - & 
            (leaching(:,m,iammonium) + nitrification(:,m,i_nh4_to_no3) + nitrification(:,m,i_nh4_to_no) + &
            nitrification(:,m,i_nh4_to_n2o)), plant_uptake(:,m,iammonium))

       plant_uptake(:,m,initrate) = MIN(soil_n_min(:,m,initrate) - & 
            leaching(:,m,initrate) - denitrification(:,m,i_no3_to_nox) + nitrification(:,m,i_nh4_to_no3), &
            plant_uptake(:,m,initrate))
    
    ENDDO
  
    !
    ! 8. Loss of N through drainage and gaseous emission
    ! (the emission part should eventually be calculated in Sechiba's diffuco routines)
    !
  
    DO m = 1,nvm
  
       ! 8.1 Loss of NH4 due to evaporation of NH3
       ! NH4+ is first converted to NH3, and then it evaporates from the soil.
       ! These equations come from Li et al. (1992), Table 4, in
       ! addition to Appendix A of Zhang et al. (2002)
  
       ! Current dissociation of [NH3] to [NH4+]
       ! See Table 4 of Li et al. 1992 and Appendix A of Zhang et al. 2002
       ! log(K_NH4) - log(K_H20) = log(NH4/NH3) + pH
       ! See also formula in "DISSOCIATION CONSTANTS OF INORGANIC ACIDS AND BASES" pdf file
       ! pK_H2O = -log(K_H2O) = 14
       ! pk_NH4 = -log(K_NH4) = 9.25
       ! pK_H2O - pK_NH4 = log([NH4+]/[NH3]) + pH
       ! [NH4+]/[NH3] = 1O^(pK_H2O - pK_NH4 - pH) = 10^(4.75-pH)
       
       ! In OCN, one makes use of frac_nh3. That should be the NH3/NH4 ratio. It is defined as:
       ! frac_nh3(:) = 10.0**(4.25-pH(:)) / (1. + 10.0**(4.25-pH(:)))
       
       ! CommentS of N. Vuichard : I have several interrogations about the equation and value used
       ! in OCN. But I have also concerns about the formulation of Li et al. ... 
       ! 1/ 
       ! The formulation of Li et al. doesn't match with the formulas in 
       ! "DISSOCIATION CONSTANTS OF INORGANIC ACIDS AND BASES" or with the formulas
       ! of http://www.onlinebiochemistry.com/obj-512/Chap4-StudNotes.html
       ! To my opinion, one should replace log(K_NH4+) by log(K_NH3) in the formulation of Li et al.
       ! with the relationship pKa + pKb = pKwater (where a and b are acid and base)
       ! this leads to [NH4+]/[NH3] = 10^(pK_NH4 - pH) = 10^(9.25 - pH)
       ! 2/
       ! Wether I'm right or not about 1/, I don't understand the value used in OCN (4.25). 
       ! It should be either 4.75 or 9.25 but 4.25 looks strange
       ! 3/ 
       ! OCN used a formulation for [NH3]/[NH4+] of the type: X/(1+X) with X=[NH3]/[NH4+]
       ! This leads to X/(1+X)=([NH3]/[NH4+])/(([NH4+]/[NH4+])+([NH3]/[NH4+]))
       ! or X/(1+X) = [NH3] / ( [NH3] + [NH4+] )
       ! This means that the value stored in soil_n_min(:,:,iammonium) corresponds to the total
       ! N of both [NH4+] and [NH3]. This makes sense to my opinion. But I wonder if one should not
       ! account for this partitioning between [NH4+] and [NH3] in other processes. To check.
       ! 4/ 
       ! The X value should relate to [NH3]/[NH4+] but to my opinion the X value used 
       ! in the equation in OCN corresponds to [NH4+]/[NH3]. Is this a bug ? 
   
       ! In conclusion, I propose the formulation
       frac_nh3(:) = 10.0**(0.15*(pH(:)-pk_NH4)) / (1. + 10.0**(0.15*(pH(:)-pk_NH4)))
  
       ! This seems a patch added to OCN in order to (as mentioned in OCN) 
       ! reduced emissions at low concentration (problem of only one soil layer)
       ! high conentrations are usually associated with fertiliser events -> top layer
       ! and thus increased emission
       ! NOT ACTIVATE HERE 
       ! emm_fac(:) = 0.01
       ! WHERE(soil_n_min(:,m,iammonium).GT.0.01.AND.soil_n_min(:,m,iammonium).LE.4.)
       !      emm_fac(:) = MAX(0.01,1-exp(-(soil_n_min(:,m,iammonium)/1.75)**8)) 
       ! ENDWHERE       
       ! WHERE(soil_n_min(:,m,iammonium).GT.4.)
       !      emm_fac(:)=1.0
       ! ENDWHERE
       
  
       ! 8.2 Volatilisation of gasous species, Table 4, Li et al. 2000 I,
       !     using diffusivity of oxigen in air as a surrogate (from OCN)
       !     assumes no effect of air concentration on diffusion
       !     takes standard depth as reference
       
       ! Clay limitation
       ! F_clay_0 = 0.13
       ! F_clay_1 = -0.079
       F_clay(:) = F_clay_0 + F_clay_1 * clay(:)
  
       ! In OCN, one used formulation of the type
       ! emission(:,m,inox-1) = & 
       !       MIN( d_ox(:) * soil_n_min(:,m,inox) * (0.13-0.079*clay(:)) * dt / z_decomp
       ! It should have the unit d_ox * soil_n_min * dt / z_decomp
       !                         m2 day-1 * gN m-2 * day / m
       !                         gN / m which is not homogeneous with the unit expected (gn m-2)
       ! To my opinion, one should not use soil_n_min (gN m-2) but a volumetric concentration (gN m-3)
       ! But I'm not clear what is the appropriate volume to consider (volume of soil ?)
  
       ! NH4 emission (gN m-2 per time step)
       emission(:,m,iammonium) = D_s(:,m) * emm_fac * frac_nh3(:) * soil_n_min(:,m,iammonium) / zmaxh &
            * F_clay(:) / z_decomp * dt
            
       ! NO NO3 emission 
       emission(:,m,initrate) = 0.
  
       ! NO2 emission (gN m-2 per time step)
       emission(:,m,inox) = D_s(:,m) * soil_n_min(:,m,inox) / zmaxh * F_clay(:) / z_decomp * dt
  
       ! N2O emission (gN m-2 per time step)
       emission(:,m,initrous) = D_s(:,m) * soil_n_min(:,m,initrous) / zmaxh * F_clay(:) / z_decomp * dt
                   
       ! N2 emission (gN m-2 per time step)
       emission(:,m,idinitro) = D_s(:,m) * soil_n_min(:,m,idinitro) / zmaxh * F_clay(:) / z_decomp * dt
              
    ENDDO
    emission(:,:,iammonium) = MIN(emission(:,:,iammonium), &
         soil_n_min(:,:,iammonium) - nitrification(:,:,i_nh4_to_no3) &
         - nitrification(:,:,i_nh4_to_no) - nitrification(:,:,i_nh4_to_n2o) &
         - leaching(:,:,iammonium))  

    emission(:,:,inox) = MIN(emission(:,:,inox), &
         soil_n_min(:,:,inox) + nitrification(:,:,i_nh4_to_no) & 
         + denitrification(:,:,i_no3_to_nox) - denitrification(:,:,i_nox_to_n2o))

    emission(:,:,initrous) = MIN(emission(:,:,initrous), &
         soil_n_min(:,:,initrous) + nitrification(:,:,i_nh4_to_n2o) & 
         + denitrification(:,:,i_nox_to_n2o) - denitrification(:,:,i_n2o_to_n2))

    emission(:,:,idinitro) = MIN(emission(:,:,idinitro), &
         soil_n_min(:,:,idinitro)  + denitrification(:,:,i_n2o_to_n2))

    !
    ! 9. Update pools
    !
  
    ! 9.1 update pools of nitrogen in the soil from nitrification and 
    ! denitrification, plant uptake, leaching and volatile emissions,
    ! desorption from clay, and net mineralisation
    !
    ! In my opinion, for a better consistency, I would recommand to consider the leaching separately 
    ! when calculating the ammonium and nitrate budget. Leaching is calculated from sechiba
    ! Might be better to remove leaching at the top of the routine especially due to the later calculation of
    ! NH4+ and NO3- concentration that will vary with the soil water content
    ! Let's imagine that from one time step to another, the change in soil water content is only due to leaching
    ! We don't want that the NH4+ and NO3- concentration vary from one time step to the other. The best way to avoid
    ! this is to remove first the leaching from the NH4+ and NO3- pools
    ! THIS IS NOT DONE YET

    IF(printlev>=4)THEN
       WRITE(numout,*) 'CHECK values before update'
       WRITE(numout,*) 'nitrification ',nitrification(test_grid,test_pft,:)
       WRITE(numout,*) 'denitrification ',denitrification(test_grid,test_pft,:)
       WRITE(numout,*) 'leaching ',leaching(test_grid,test_pft,:)
       WRITE(numout,*) 'emission ',emission(test_grid,test_pft,:)
       WRITE(numout,*) 'plant_uptake ',plant_uptake(test_grid,test_pft,:)
       WRITE(numout,*) 'mineralisation ',mineralisation(test_grid,test_pft)
       WRITE(numout,*) 'immob ',immob(test_grid,test_pft)
    ENDIF

    soil_n_min(:,:,iammonium) = soil_n_min(:,:,iammonium) + n_adsorbed(:,:) & 
         - nitrification(:,:,i_nh4_to_no3) - nitrification(:,:,i_nh4_to_no) - nitrification(:,:,i_nh4_to_n2o) &
         - leaching(:,:,iammonium) - emission(:,:,iammonium) &
         + mineralisation(:,:) + immob(:,:) - plant_uptake(:,:,iammonium)
  
    soil_n_min(:,:,initrate) = soil_n_min(:,:,initrate) + nitrification(:,:,i_nh4_to_no3) & 
         - denitrification(:,:,i_no3_to_nox) - leaching(:,:,initrate) &
         - plant_uptake(:,:,initrate)
  
    soil_n_min(:,:,inox) =  soil_n_min(:,:,inox) + nitrification(:,:,i_nh4_to_no) & 
         + denitrification(:,:,i_no3_to_nox) - denitrification(:,:,i_nox_to_n2o) - emission(:,:,inox)
  
    soil_n_min(:,:,initrous) = soil_n_min(:,:,initrous) + nitrification(:,:,i_nh4_to_n2o) & 
         + denitrification(:,:,i_nox_to_n2o) - denitrification(:,:,i_n2o_to_n2) - emission(:,:,initrous)
  
    soil_n_min(:,:,idinitro) = soil_n_min(:,:,idinitro)  + denitrification(:,:,i_n2o_to_n2) &
         - emission(:,:,idinitro)  


    IF(printlev>=4)THEN
       WRITE(numout,*) 'CHECK values after update'
       WRITE(numout,*) 'soil_n_min ',soil_n_min(test_grid,test_pft,:)
    ENDIF
    ! 9.2 add nitrogen either from deposition (read from fields or prescribed in run.def) as
    ! well as BNF, needs to be revised...
    ! iatm_ammo=1
    ! iatm_nitr=2
    ! ibnf=3
    ! ifert=4
    DO m=1,nvm
       ! Deposition of NHx and NOy
       WHERE(veget_max(:,m).GT.min_stomate.AND.som(:,iactive,m,icarbon).GT.min_stomate) 
          soil_n_min(:,m,iammonium) = soil_n_min(:,m,iammonium) & 
               + input(:,m,iammonium)*dt
          soil_n_min(:,m,initrate) = soil_n_min(:,m,initrate) & 
               + input(:,m,initrate)*dt
       ENDWHERE
       WHERE(veget_max(:,m).GT.min_stomate.AND.som(:,iactive,m,icarbon).LE.min_stomate) 
          leaching(:,m,iammonium)=leaching(:,m,iammonium) + &
              input(:,m,iammonium)*dt
          leaching(:,m,initrate)=leaching(:,m,initrate) + &
              input(:,m,initrate)*dt
       ENDWHERE
       ! BNF
       IF ( natural(m) ) THEN  
          ! in the presence of a organic soil component assume that there is also BNF
          ! as long as plant available nitrogen is not too high
          WHERE(som(:,iactive,m,icarbon).GT.min_stomate.AND. & 
               soil_n_min(:,m,iammonium)+soil_n_min(:,m,initrate).LT.max_soil_n_bnf(m))
             soil_n_min(:,m,iammonium) = soil_n_min(:,m,iammonium) + input(:,m,ibnf)*dt
          ENDWHERE
       ENDIF
       ! Fertiliser use for agriculture, no BNF, that's already accounted for in fertil
       ! using the average global ratio of ammonium to nitrate to
       ! separate the two species
       ! ratio_nh4_fert = 7./8.
       WHERE(veget_max(:,m).GT.min_stomate)   
          soil_n_min(:,m,iammonium) = soil_n_min(:,m,iammonium) &
               + input(:,m,ifert)*ratio_nh4_fert*dt 
          soil_n_min(:,m,initrate) = soil_n_min(:,m,initrate) & 
               + input(:,m,ifert)*(1.-ratio_nh4_fert)*dt
          ! Application of manure (from grazing and the one produced during
          ! storage)
          ! Assumption : nitrate = non tan and ammonium = tan 
          soil_n_min(:,m,iammonium) = soil_n_min(:,m,iammonium) &
               +  input(:,m,imanure_tan)*dt + input(:,m,igraz_tan)*dt
          soil_n_min(:,m,initrate) = soil_n_min(:,m,initrate) &
               + input(:,m,imanure_org)*dt + input(:,m,igraz_org)*dt
       ENDWHERE
    ENDDO

    IF(printlev>=4)THEN
       WRITE(numout,*) 'CHECK values after N input'
       WRITE(numout,*) 'soil_n_min ',soil_n_min(test_grid,test_pft,:)
    ENDIF

    ! 10.  Write output values
    CALL histwrite_p (hist_id_stomate, 'N_UPTAKE_NH4', itime, &
         plant_uptake(:,:,iammonium)/dt, npts*nvm, horipft_index)
    CALL histwrite_p (hist_id_stomate, 'N_UPTAKE_NO3', itime, &
         plant_uptake(:,:,initrate)/dt, npts*nvm, horipft_index)
    CALL histwrite_p (hist_id_stomate, 'N_MINERALISATION', itime, &
         mineralisation(:,:)/dt, npts*nvm, horipft_index)
    CALL histwrite_p (hist_id_stomate, 'SOIL_NH4', itime, &
         soil_n_min(:,:,iammonium), npts*nvm, horipft_index)
    CALL histwrite_p (hist_id_stomate, 'SOIL_NO3', itime, &
         soil_n_min(:,:,initrate), npts*nvm, horipft_index)
    CALL histwrite_p (hist_id_stomate, 'SOIL_NOX', itime, &
         soil_n_min(:,:,inox), npts*nvm, horipft_index)
    CALL histwrite_p (hist_id_stomate, 'SOIL_N2O', itime, &
         soil_n_min(:,:,initrous), npts*nvm, horipft_index)
    CALL histwrite_p (hist_id_stomate, 'SOIL_N2', itime, &
         soil_n_min(:,:,idinitro), npts*nvm, horipft_index)
    CALL histwrite_p (hist_id_stomate, 'SOIL_P_OX', itime, &
         p_O2(:,:), npts*nvm, horipft_index)
    CALL histwrite_p (hist_id_stomate, 'BACT', itime, &
         bact(:,:), npts*nvm, horipft_index)
    CALL histwrite_p (hist_id_stomate, 'NH3_EMISSION', itime, &
         emission(:,:,iammonium)/dt, npts*nvm, horipft_index)
    CALL histwrite_p (hist_id_stomate, 'NOX_EMISSION', itime, &
         emission(:,:,inox)/dt, npts*nvm, horipft_index)
    CALL histwrite_p (hist_id_stomate, 'N2O_EMISSION', itime, &
         emission(:,:,initrous)/dt, npts*nvm, horipft_index)
    CALL histwrite_p (hist_id_stomate, 'N2_EMISSION', itime, &
         emission(:,:,idinitro)/dt, npts*nvm, horipft_index)
    CALL histwrite_p (hist_id_stomate, 'NH4_LEACHING', itime, &
         leaching(:,:,iammonium)/dt, npts*nvm, horipft_index)
    CALL histwrite_p (hist_id_stomate, 'NO3_LEACHING', itime, &
         leaching(:,:,initrate)/dt, npts*nvm, horipft_index)
    CALL histwrite_p (hist_id_stomate, 'NITRIFICATION', itime, &
         nitrification(:,:,i_nh4_to_no3), npts*nvm, horipft_index)
    CALL histwrite_p (hist_id_stomate, 'DENITRIFICATION', itime, &
         denitrification(:,:,i_n2o_to_n2), npts*nvm, horipft_index)
    CALL histwrite_p (hist_id_stomate, 'NHX_DEPOSITION', itime, &
         input(:,:,iammonium), npts*nvm, horipft_index)
    CALL histwrite_p (hist_id_stomate, 'NOX_DEPOSITION', itime, &
         input(:,:,initrate), npts*nvm, horipft_index)
    CALL histwrite_p (hist_id_stomate, 'BNF', itime, &
         input(:,:,ibnf), npts*nvm, horipft_index)
    CALL histwrite_p (hist_id_stomate, 'N_FERTILISER', itime, &
         input(:,:,ifert), npts*nvm, horipft_index)  
    CALL histwrite_p (hist_id_stomate, 'N_MANURE', itime, &
         input(:,:,imanure), npts*nvm, horipft_index)  

    CALL xios_orchidee_send_field("N_UPTAKE_NH4",plant_uptake(:,:,iammonium)/dt)
    CALL xios_orchidee_send_field("N_UPTAKE_NO3",plant_uptake(:,:,initrate)/dt)
    CALL xios_orchidee_send_field("N_MINERALISATION",mineralisation(:,:)/dt)
    CALL xios_orchidee_send_field("SOIL_NH4",soil_n_min(:,:,iammonium))
    CALL xios_orchidee_send_field("SOIL_NO3",soil_n_min(:,:,initrate))
    CALL xios_orchidee_send_field("SOIL_NOX",soil_n_min(:,:,inox))
    CALL xios_orchidee_send_field("SOIL_N2O",soil_n_min(:,:,initrous))
    CALL xios_orchidee_send_field("SOIL_N2",soil_n_min(:,:,idinitro))
    CALL xios_orchidee_send_field("SOIL_P_OX",p_O2(:,:))
    CALL xios_orchidee_send_field("BACT",bact(:,:))
    CALL xios_orchidee_send_field("NH3_EMISSION",emission(:,:,iammonium)/dt)
    CALL xios_orchidee_send_field("NOX_EMISSION",emission(:,:,inox)/dt)
    CALL xios_orchidee_send_field("N2O_EMISSION",emission(:,:,initrous)/dt)
    CALL xios_orchidee_send_field("N2_EMISSION",emission(:,:,idinitro)/dt)
    CALL xios_orchidee_send_field("NH4_LEACHING",leaching(:,:,iammonium)/dt)
    CALL xios_orchidee_send_field("NO3_LEACHING",leaching(:,:,initrate)/dt)
    CALL xios_orchidee_send_field("NITRIFICATION",nitrification(:,:,i_nh4_to_no3))
    CALL xios_orchidee_send_field("DENITRIFICATION",denitrification(:,:,i_n2o_to_n2))
    CALL xios_orchidee_send_field("NHX_DEPOSITION",input(:,:,iammonium))
    CALL xios_orchidee_send_field("NOX_DEPOSITION",input(:,:,initrate))
    CALL xios_orchidee_send_field("BNF",input(:,:,ibnf))
    CALL xios_orchidee_send_field("N_FERTILISER",input(:,:,ifert))  
    CALL xios_orchidee_send_field("N_MANURE",input(:,:,imanure))  
  END SUBROUTINE nitrogen_dynamics



!! ================================================================================================================================
!!  FUNCTION   : control_temp_func
!!
!>\BRIEF        : Unclear.
!!
!! DESCRIPTION	: Unable to find where this comes from.
!!   Referenced by Zaehle and Friend (2010) in the Appendix,
!!   which points to Krinner et al (2005), but the closest I found
!!   there was Eq. A32, which is simillar but does not
!!   mention Q10 at all.
!!
!! RECENT CHANGE(S): 
!! 
!! MAIN OUTPUTS VARIABLE(S): 
!!
!! REFERENCE(S)   :
!! - S. Zaehle and A. D. Friend (2010), Carbon and nitrogen cycle dynamics in the
!!   O-CN land surface model: 1. Model description, site-scale evaluation, and 
!!   sensitivity to parameter estimates. Global Biogeochem. Cycles, 24, GB1005, 
!!   doi:10.1029/2009GB003521.
!! - Krinner G., N. Viovy, N. de Noblet-Ducoudre, J. Ogee, J. Polcher, P. Friedlingstein, 
!!   P. Ciais, S. Sitch, and I. C. Prentice (2005), A dynamic global vegetation model 
!!   for studies of the coupled atmosphere-biosphere system, Global Biogeochemical 
!!   Cycles, 19, doi.:10.1029/2003/GB002199.
!! 
!! FLOWCHART    :
!!
!_ ================================================================================================================================
  FUNCTION control_temp_func (npts, temp_in) RESULT (tempfunc_result)

  !! 0. Variable and parameter declaration
    
    !! 0.1 Input variables
    INTEGER(i_std), INTENT(in)                 :: npts            !! Domain size - number of land pixels (unitless)
    REAL(r_std), DIMENSION(npts), INTENT(in)   :: temp_in         !! Temperature (K)

    !! 0.2 Output variables
    REAL(r_std), DIMENSION(npts)               :: tempfunc_result !! Temperature control factor (0-1, unitless)

    !! 0.3 Modified variables

    !! 0.4 Local variables

!_ ================================================================================================================================

    tempfunc_result(:) = exp( soil_Q10_uptake * ( temp_in(:) - (ZeroCelsius+tsoil_ref)) / Q10 )
    tempfunc_result(:) = MIN( un, tempfunc_result(:) )

  END FUNCTION control_temp_func


END MODULE stomate_som_dynamics
